@@ -41,7 +41,6 @@ package org.jruby;
 import org.jruby.exceptions.MainExitException;
 import org.jruby.exceptions.JumpException;
 import org.jruby.exceptions.RaiseException;
-import org.jruby.exceptions.SignalException;
 import org.jruby.exceptions.ThreadKill;
 import org.jruby.main.DripMain;
 import org.jruby.platform.Platform;
@@ -53,25 +52,16 @@ import org.jruby.util.cli.OutputStrings;
 import org.jruby.util.log.Logger;
 import org.jruby.util.log.LoggerFactory;
 
-import java.io.File;
-import java.io.FileDescriptor;
-import java.io.FileInputStream;
-import java.io.FileNotFoundException;
-import java.io.IOException;
-import java.io.InputStream;
-import java.io.PrintStream;
+import java.io.*;
 import java.lang.management.ManagementFactory;
 import java.lang.management.RuntimeMXBean;
-import java.lang.reflect.InvocationHandler;
-import java.lang.reflect.Method;
-import java.lang.reflect.Proxy;
+import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.HashSet;
 import java.util.Properties;
-import java.util.concurrent.atomic.AtomicBoolean;
 
 /**
  * Class used to launch the interpreter.
@@ -438,7 +428,7 @@ public class Main {
         final ThreadContext context = runtime.getCurrentContext();
         final IRubyObject $ex = context.getErrorInfo();
         try {
-            runtime.parseFromMain(in, filename);
+            runtime.parseFromMain(filename, in);
             config.getOutput().println("Syntax OK");
             return true;
         } catch (RaiseException re) {
@@ -471,11 +461,47 @@ public class Main {
 
     private void doPrintUsage(boolean force) {
         if (config.getShouldPrintUsage() || force) {
-            config.getOutput().print(OutputStrings.getBasicUsageHelp());
-            config.getOutput().print(OutputStrings.getFeaturesHelp());
+            String rubyPager = getRubyPagerEnv();
+
+            // Do not want to boot native subsystem here, so we do best guess based on System.console. It will be
+            // non-null only if both STDIN and STDOUT are tty.
+            boolean tty = System.console() != null;
+
+            if (rubyPager == null) {
+                config.getOutput().print(OutputStrings.getBasicUsageHelp(tty));
+                config.getOutput().print(OutputStrings.getFeaturesHelp(tty));
+            } else {
+                try {
+                    ProcessBuilder builder = new ProcessBuilder(rubyPager);
+                    builder.environment().put("LESS", "-R");
+
+                    builder.redirectOutput(ProcessBuilder.Redirect.INHERIT);
+                    builder.redirectError(ProcessBuilder.Redirect.INHERIT);
+
+                    Process process = builder.start();
+                    OutputStream in = process.getOutputStream();
+
+                    String fullHelp = OutputStrings.getBasicUsageHelp(tty) + OutputStrings.getFeaturesHelp(tty);
+                    in.write(fullHelp.getBytes(StandardCharsets.UTF_8));
+
+                    in.flush();
+                    in.close();
+                    process.waitFor();
+                } catch (InterruptedException | IOException e) {
+                    e.printStackTrace();
+                }
+            }
         }
     }
 
+    private String getRubyPagerEnv() {
+        String rubyPager = System.getenv("RUBY_PAGER");
+        if (rubyPager == null)
+            rubyPager = System.getenv("PAGER");
+
+        return rubyPager;
+    }
+
     private void doShowCopyright() {
         if (config.isShowCopyright()) {
             config.getOutput().println(OutputStrings.getCopyrightString());
@@ -532,19 +558,6 @@ public class Main {
         boolean isatty = runtime.getPosix().isatty(FileDescriptor.err);
 
         System.err.print(traceType.printBacktrace(raisedException, isatty));
-
-        Set<Object> shownCauses = new HashSet<Object>();
-        shownCauses.add(raisedException);
-
-        for (
-                Object cause = raisedException.getCause();
-                cause != null && cause instanceof RubyException && !shownCauses.contains(cause);
-                cause = ((RubyException) cause).getCause()) {
-
-            System.err.print(traceType.printBacktrace((RubyException) cause, isatty));
-            shownCauses.add(cause);
-        }
-
         return 1;
     }
 
