@@ -1,7 +1,5 @@
 package org.jruby;
 
-import org.jruby.ext.stringio.StringIO;
-
 /**
  * Flags used by RubyBasicObject descendants.
  */
@@ -13,6 +11,9 @@ public interface ObjectFlags {
     int FALSE_F = registry.newFlag(RubyBasicObject.class);
     int NIL_F = registry.newFlag(RubyBasicObject.class);
     int FROZEN_F = registry.newFlag(RubyBasicObject.class);
+
+    // Deprecated and unused but don't move due to checks elsewhere for the following flags
+    @Deprecated
     int TAINTED_F = registry.newFlag(RubyBasicObject.class);
 
     int CACHEPROXY_F = registry.newFlag(RubyModule.class);
@@ -25,12 +26,11 @@ public interface ObjectFlags {
     int CR_7BIT_F    = registry.newFlag(RubyString.class);
     int CR_VALID_F   = registry.newFlag(RubyString.class);
 
-    int STRIO_READABLE = registry.newFlag(StringIO.class);
-    int STRIO_WRITABLE = registry.newFlag(StringIO.class);
-
     int MATCH_BUSY = registry.newFlag(RubyMatchData.class);
 
     int COMPARE_BY_IDENTITY_F = registry.newFlag(RubyHash.class);
     int KEYWORD_REST_ARGUMENTS_F = registry.newFlag(RubyHash.class);
     int PROCDEFAULT_HASH_F = registry.newFlag(RubyHash.class);
+    int KEYWORD_ARGUMENTS_F = registry.newFlag(RubyHash.class);
+    int RUBY2_KEYWORD_F = registry.newFlag(RubyHash.class);
 }
