@@ -26,38 +26,6 @@
 
 package org.jruby;
 
-import static org.jruby.util.Numeric.f_abs;
-import static org.jruby.util.Numeric.f_add;
-import static org.jruby.util.Numeric.f_arg;
-import static org.jruby.util.Numeric.f_conjugate;
-import static org.jruby.util.Numeric.f_denominator;
-import static org.jruby.util.Numeric.f_div;
-import static org.jruby.util.Numeric.f_equal;
-import static org.jruby.util.Numeric.f_exact_p;
-import static org.jruby.util.Numeric.f_expt;
-import static org.jruby.util.Numeric.f_gt_p;
-import static org.jruby.util.Numeric.f_inspect;
-import static org.jruby.util.Numeric.f_lcm;
-import static org.jruby.util.Numeric.f_mul;
-import static org.jruby.util.Numeric.f_negate;
-import static org.jruby.util.Numeric.f_negative_p;
-import static org.jruby.util.Numeric.f_numerator;
-import static org.jruby.util.Numeric.f_one_p;
-import static org.jruby.util.Numeric.f_quo;
-import static org.jruby.util.Numeric.f_real_p;
-import static org.jruby.util.Numeric.f_reciprocal;
-import static org.jruby.util.Numeric.f_sub;
-import static org.jruby.util.Numeric.f_to_f;
-import static org.jruby.util.Numeric.f_to_i;
-import static org.jruby.util.Numeric.f_to_r;
-import static org.jruby.util.Numeric.f_to_s;
-import static org.jruby.util.Numeric.f_xor;
-import static org.jruby.util.Numeric.f_zero_p;
-import static org.jruby.util.Numeric.k_exact_p;
-import static org.jruby.util.Numeric.k_inexact_p;
-import static org.jruby.util.Numeric.num_pow;
-import static org.jruby.util.Numeric.safe_mul;
-
 import org.jcodings.specific.ASCIIEncoding;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
@@ -66,18 +34,26 @@ import org.jruby.exceptions.RaiseException;
 import org.jruby.runtime.Arity;
 import org.jruby.runtime.CallSite;
 import org.jruby.runtime.ClassIndex;
+import org.jruby.runtime.Helpers;
 import org.jruby.runtime.JavaSites;
+import org.jruby.runtime.ObjectMarshal;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
+import org.jruby.runtime.marshal.MarshalStream;
+import org.jruby.runtime.marshal.UnmarshalStream;
 import org.jruby.util.ByteList;
 import org.jruby.util.Numeric;
 import org.jruby.util.TypeConverter;
 
+import java.io.IOException;
+import java.nio.ByteBuffer;
+import java.util.Arrays;
 import java.util.function.BiFunction;
 
 import static org.jruby.runtime.Helpers.invokedynamic;
 import static org.jruby.runtime.invokedynamic.MethodNames.HASH;
+import static org.jruby.util.Numeric.*;
 import static org.jruby.util.RubyStringBuilder.str;
 import static org.jruby.util.RubyStringBuilder.types;
 
@@ -89,8 +65,8 @@ public class RubyComplex extends RubyNumeric {
 
     public static RubyClass createComplexClass(Ruby runtime) {
         final String[] UNDEFINED = new String[]{
-                "<", "<=", "<=>", ">", ">=",
-                "between?", "divmod", "floor", "ceil", "modulo",
+                "<", "<=", ">", ">=", "between?", "clamp",
+                "%", "div", "divmod", "floor", "ceil", "modulo", "remainder",
                 "round", "step", "truncate", "positive?", "negative?"
         };
 
@@ -101,6 +77,7 @@ public class RubyComplex extends RubyNumeric {
         
         complexc.kindOf = new RubyModule.JavaClassKindOf(RubyComplex.class);
 
+        complexc.setMarshal(COMPLEX_MARSHAL);
         complexc.defineAnnotatedMethods(RubyComplex.class);
 
         complexc.getSingletonClass().undefineMethod("allocate");
@@ -115,14 +92,10 @@ public class RubyComplex extends RubyNumeric {
         return complexc;
     }
 
-    /** internal
-     * 
-     */
     private RubyComplex(Ruby runtime, RubyClass clazz, IRubyObject real, IRubyObject image) {
         super(runtime, clazz);
         this.real = real;
         this.image = image;
-        this.flags |= FROZEN_F;
     }
 
     private RubyComplex(Ruby runtime, RubyClass clazz) {
@@ -132,7 +105,6 @@ public class RubyComplex extends RubyNumeric {
 
         this.real = zero;
         this.image = zero;
-        this.flags |= FROZEN_F;
     }
 
     @Override
@@ -140,22 +112,31 @@ public class RubyComplex extends RubyNumeric {
         return ClassIndex.COMPLEX;
     }
 
+    /** internal
+     *
+     */
+    private static RubyComplex newComplexInternal(Ruby runtime, RubyClass clazz, IRubyObject x, IRubyObject y) {
+        RubyComplex ret = new RubyComplex(runtime, clazz, x, y);
+        ret.setFrozen(true);
+        return ret;
+    }
+
     /** rb_complex_raw
      * 
      */
     public static RubyComplex newComplexRaw(Ruby runtime, IRubyObject x, IRubyObject y) {
-        return new RubyComplex(runtime, runtime.getComplex(), x, y);
+        return newComplexInternal(runtime, runtime.getComplex(), x, y);
     }
 
     /** rb_complex_raw1
      * 
      */
     public static RubyComplex newComplexRaw(Ruby runtime, IRubyObject x) {
-        return new RubyComplex(runtime, runtime.getComplex(), x, RubyFixnum.zero(runtime));
+        return newComplexRaw(runtime, x, RubyFixnum.zero(runtime));
     }
 
     public static RubyComplex newComplexRawImage(Ruby runtime, IRubyObject image) {
-        return new RubyComplex(runtime, runtime.getComplex(), RubyFixnum.zero(runtime), image);
+        return newComplexRaw(runtime, RubyFixnum.zero(runtime), image);
     }
 
     /** rb_complex_new1
@@ -191,14 +172,16 @@ public class RubyComplex extends RubyNumeric {
      */
     static RubyNumeric newComplex(ThreadContext context, RubyClass clazz, IRubyObject x, IRubyObject y) {
         assert !(x instanceof RubyComplex);
-        return canonicalizeInternal(context, clazz, x, y);
+        RubyNumeric ret = canonicalizeInternal(context, clazz, x, y);
+        ret.setFrozen(true);
+        return ret;
     }
     
     /** f_complex_new_bang2
      * 
      */
     static RubyComplex newComplexBang(ThreadContext context, RubyClass clazz, RubyNumeric x, RubyNumeric y) {
-        return new RubyComplex(context.runtime, clazz, x, y);
+        return newComplexInternal(context.runtime, clazz, x, y);
     }
 
     /** f_complex_new_bang1
@@ -285,14 +268,14 @@ public class RubyComplex extends RubyNumeric {
     @JRubyMethod(name = "new!", meta = true, visibility = Visibility.PRIVATE)
     public static IRubyObject newInstanceBang(ThreadContext context, IRubyObject recv, IRubyObject real) {
         if (!(real instanceof RubyNumeric)) real = f_to_i(context, real);
-        return new RubyComplex(context.runtime, (RubyClass) recv, real, RubyFixnum.zero(context.runtime));
+        return newComplexInternal(context.runtime, (RubyClass) recv, real, RubyFixnum.zero(context.runtime));
     }
 
     @JRubyMethod(name = "new!", meta = true, visibility = Visibility.PRIVATE)
     public static IRubyObject newInstanceBang(ThreadContext context, IRubyObject recv, IRubyObject real, IRubyObject image) {
         if (!(real instanceof RubyNumeric)) real = f_to_i(context, real);
         if (!(image instanceof RubyNumeric)) image = f_to_i(context, image);
-        return new RubyComplex(context.runtime, (RubyClass) recv, real, image);
+        return newComplexInternal(context.runtime, (RubyClass) recv, real, image);
     }
 
     /** nucomp_canonicalization
@@ -335,23 +318,23 @@ public class RubyComplex extends RubyNumeric {
         boolean realComplex = real instanceof RubyComplex;
         boolean imageComplex = image instanceof RubyComplex;
         if (!realComplex && !imageComplex) {
-            return new RubyComplex(context.runtime, clazz, real, image);
+            return newComplexInternal(context.runtime, clazz, real, image);
         }
         if (!realComplex) {
             RubyComplex complex = (RubyComplex)image;
-            return new RubyComplex(context.runtime, clazz,
+            return newComplexInternal(context.runtime, clazz,
                                    f_sub(context, real, complex.image),
                                    f_add(context, RubyFixnum.zero(context.runtime), complex.real));
         }
         if (!imageComplex) {
             RubyComplex complex = (RubyComplex)real;
-            return new RubyComplex(context.runtime, clazz,
+            return newComplexInternal(context.runtime, clazz,
                                    complex.real,
                                    f_add(context, complex.image, image));
         } else {
             RubyComplex complex1 = (RubyComplex)real;
             RubyComplex complex2 = (RubyComplex)image;
-            return new RubyComplex(context.runtime, clazz,
+            return newComplexInternal(context.runtime, clazz,
                                    f_sub(context, complex1.real, complex2.image),
                                    f_add(context, complex1.image, complex2.real));
         }
@@ -414,17 +397,19 @@ public class RubyComplex extends RubyNumeric {
     /** nucomp_s_polar
      *
      */
-    @JRubyMethod(name = "polar", meta = true, required = 1, optional = 1)
+    @JRubyMethod(name = "polar", meta = true, required = 1, optional = 1, checkArity = false)
     public static IRubyObject polar(ThreadContext context, IRubyObject clazz, IRubyObject... args) {
+        int argc = Arity.checkArgumentCount(context, args, 1, 2);
+
         IRubyObject abs = args[0];
-        IRubyObject arg;
-        if (args.length < 2) {
-            arg = RubyFixnum.zero(context.runtime);
-        } else {
-            arg = args[1];
-        }
+        IRubyObject arg = argc < 2 ? RubyFixnum.zero(context.runtime) : args[1];
+
         realCheck(context, abs, true);
         realCheck(context, arg, true);
+
+        if (abs instanceof RubyComplex) abs = ((RubyComplex) abs).getReal();
+        if (arg instanceof RubyComplex) arg = ((RubyComplex) arg).getReal();
+
         return f_complex_polar(context, (RubyClass) clazz, abs, arg);
     }
 
@@ -463,6 +448,7 @@ public class RubyComplex extends RubyNumeric {
      */
     @JRubyMethod(name = "convert", meta = true, visibility = Visibility.PRIVATE)
     public static IRubyObject convert(ThreadContext context, IRubyObject recv, IRubyObject arg) {
+        if (arg instanceof RubyComplex) return arg;
         return convertCommon(context, recv, arg, null, true);
     }
 
@@ -473,21 +459,19 @@ public class RubyComplex extends RubyNumeric {
     public static IRubyObject convert(ThreadContext context, IRubyObject recv, IRubyObject a1, IRubyObject a2) {
         Ruby runtime = context.runtime;
 
-        boolean raise = true;
+        IRubyObject maybeKwargs = ArgsUtil.getOptionsArg(runtime, a2, false);
 
-        if (a2 != null) {
-            IRubyObject maybeKwargs = ArgsUtil.getOptionsArg(runtime, a2, false);
-
-            if (!maybeKwargs.isNil()) {
-                a2 = null;
-
-                IRubyObject exception = ArgsUtil.extractKeywordArg(context, "exception", (RubyHash) maybeKwargs);
-
-                raise = exception.isNil() ? true : exception.isTrue();
-            }
+        if (maybeKwargs.isNil()) {
+            return convertCommon(context, recv, a1, a2, true);
         }
 
-        return convertCommon(context, recv, a1, a2, raise);
+        IRubyObject exception = ArgsUtil.extractKeywordArg(context, "exception", (RubyHash) maybeKwargs);
+        if (exception instanceof RubyBoolean) {
+            if (a1 instanceof RubyComplex) return a1;
+            return convertCommon(context, recv, a1, null, exception.isTrue());
+        }
+
+        throw runtime.newArgumentError("`Complex': expected true or false as exception: " + exception); 
     }
 
     /** nucomp_s_convert
@@ -498,17 +482,17 @@ public class RubyComplex extends RubyNumeric {
         Ruby runtime = context.runtime;
 
         IRubyObject maybeKwargs = ArgsUtil.getOptionsArg(runtime, kwargs, false);
-        boolean raise;
 
         if (maybeKwargs.isNil()) {
             throw runtime.newArgumentError("convert", 3, 1, 2);
         }
 
         IRubyObject exception = ArgsUtil.extractKeywordArg(context, "exception", (RubyHash) maybeKwargs);
+        if (exception instanceof RubyBoolean) {
+            return convertCommon(context, recv, a1, a2, exception.isTrue());
+        }
 
-        raise = exception.isNil() ? true : exception.isTrue();
-
-        return convertCommon(context, recv, a1, a2, raise);
+        throw runtime.newArgumentError("`Complex': expected true or false as exception: " + exception); 
     }
 
     // MRI: nucomp_s_convert
@@ -580,6 +564,24 @@ public class RubyComplex extends RubyNumeric {
         return newInstance(context, recv, a1, a2, true);
     }
 
+    // MRI: nucomp_real_p
+    private boolean nucomp_real_p(ThreadContext context) {
+        return f_zero_p(context, image);
+    }
+
+    @JRubyMethod(name="<=>")
+    public IRubyObject op_cmp(ThreadContext context, IRubyObject other) {
+        if (nucomp_real_p(context) && k_numeric_p(other)) {
+            if (other instanceof RubyComplex && ((RubyComplex) other).nucomp_real_p(context)) {
+                return real.callMethod(context, "<=>", ((RubyComplex) other).real);
+            } else if (f_real_p(context, other)) {
+                return real.callMethod(context, "<=>", other);
+            }
+        }
+
+        return context.nil;
+    }
+
     /** nucomp_real
      * 
      */
@@ -862,12 +864,12 @@ public class RubyComplex extends RubyNumeric {
      */
     @JRubyMethod(name = "coerce")
     public IRubyObject coerce(ThreadContext context, IRubyObject other) {
+        if (other instanceof RubyComplex) return context.runtime.newArray(other, this);
+
         if (other instanceof RubyNumeric && f_real_p(context, other)) {
             return context.runtime.newArray(newComplexBang(context, getMetaClass(), (RubyNumeric) other), this);
         }
 
-        if (other instanceof RubyComplex) return context.runtime.newArray(other, this);
-
         Ruby runtime = context.runtime;
         throw runtime.newTypeError(str(runtime, types(runtime, other.getMetaClass()), " can't be coerced into ", types(runtime, getMetaClass())));
     }
@@ -993,7 +995,11 @@ public class RubyComplex extends RubyNumeric {
      */
     @JRubyMethod(name = "hash")
     public IRubyObject hash(ThreadContext context) {
-        return f_xor(context, invokedynamic(context, real, HASH), invokedynamic(context, image, HASH));
+        Ruby runtime = context.runtime;
+        long realHash = RubyNumeric.fix2long(invokedynamic(context, real, HASH));
+        long imageHash = RubyNumeric.fix2long(invokedynamic(context, image, HASH));
+        byte [] bytes = ByteBuffer.allocate(16).putLong(realHash).putLong(imageHash).array();
+        return RubyFixnum.newFixnum(runtime, Helpers.multAndMix(runtime.getHashSeedK0(), Arrays.hashCode(bytes)));
     }
 
     @Override
@@ -1087,7 +1093,7 @@ public class RubyComplex extends RubyNumeric {
     /** nucomp_marshal_dump
      * 
      */
-    @JRubyMethod(name = "marshal_dump")
+    @JRubyMethod(name = "marshal_dump", visibility = Visibility.PRIVATE)
     public IRubyObject marshal_dump(ThreadContext context) {
         RubyArray dump = context.runtime.newArray(real, image);
         if (hasVariables()) dump.syncVariables(this);
@@ -1097,7 +1103,7 @@ public class RubyComplex extends RubyNumeric {
     /** nucomp_marshal_load
      * 
      */
-    @JRubyMethod(name = "marshal_load")
+    @JRubyMethod(name = "marshal_load", visibility = Visibility.PRIVATE)
     public IRubyObject marshal_load(ThreadContext context, IRubyObject arg) {
         RubyArray load = arg.convertToArray();
         real = load.size() > 0 ? load.eltInternal(0) : context.nil;
@@ -1107,6 +1113,26 @@ public class RubyComplex extends RubyNumeric {
         return this;
     }
 
+    private static final ObjectMarshal COMPLEX_MARSHAL = new ObjectMarshal() {
+        @Override
+        public void marshalTo(Ruby runtime, Object obj, RubyClass type, MarshalStream marshalStream) {
+            //do nothing
+        }
+
+        @Override
+        public Object unmarshalFrom(Ruby runtime, RubyClass type,
+                                    UnmarshalStream unmarshalStream) throws IOException {
+            ThreadContext context = runtime.getCurrentContext();
+
+            RubyComplex c = (RubyComplex)RubyClass.DEFAULT_OBJECT_MARSHAL.unmarshalFrom(runtime, type, unmarshalStream);
+            c.real = c.removeInstanceVariable("@real");
+            c.image = c.removeInstanceVariable("@image");
+            c.setFrozen(true);
+
+            return c;
+        }
+    };
+
     /** nucomp_to_c
      *
      */
@@ -1151,7 +1177,7 @@ public class RubyComplex extends RubyNumeric {
     /** nucomp_rationalize
      *
      */
-    @JRubyMethod(name = "rationalize", optional = 1)
+    @JRubyMethod(name = "rationalize", optional = 1, checkArity = false)
     public IRubyObject rationalize(ThreadContext context, IRubyObject[] args) {
         if (k_inexact_p(image) || !f_zero_p(context, image)) {
             throw context.runtime.newRangeError("can't convert " + f_to_s(context, this).convertToString() + " into Rational");
@@ -1261,10 +1287,15 @@ public class RubyComplex extends RubyNumeric {
 
         final RubyFixnum zero = RubyFixnum.zero(runtime);
 
-        RubyNumeric r = convertString(context, sr, zero);
-        RubyNumeric i = convertString(context, si, zero);
+        try {
+            RubyNumeric r = convertString(context, sr, zero);
+            RubyNumeric i = convertString(context, si, zero);
 
-        return new IRubyObject[] { po ? newComplexPolar(context, r, i) : newComplexCanonicalize(context, r, i), re };
+            return new IRubyObject[]{po ? newComplexPolar(context, r, i) : newComplexCanonicalize(context, r, i), re};
+        } catch(RaiseException exception) {
+            context.setErrorInfo(context.nil);
+            return new IRubyObject[] {context.nil, str};
+        }
     }
 
     private static RubyNumeric convertString(ThreadContext context, final IRubyObject s, RubyFixnum zero) {
@@ -1276,11 +1307,19 @@ public class RubyComplex extends RubyNumeric {
         if (f_gt_p(context, s.callMethod(context, "count", RubyString.newStringShared(runtime, _eE)), zero)) {
             return (RubyNumeric) f_to_f(context, s);
         }
-        return (RubyNumeric) f_to_i(context, s);
+        return (RubyNumeric) ((RubyString) s).stringToInum(10, true);
     }
 
     // MRI: string_to_c_strict
     private static IRubyObject str_to_c_strict(ThreadContext context, RubyString str, boolean raise) {
+        str.verifyAsciiCompatible();
+
+        if (str.hasNul()) {
+            if (!raise) return context.nil;
+
+            throw context.runtime.newArgumentError("string contains null byte");
+        }
+
         IRubyObject[] ary = str_to_c_internal(context, str);
         if (ary[0] == context.nil || ary[1].convertToString().getByteList().length() > 0) {
             if (raise) {
