@@ -146,7 +146,7 @@ public class RubyFileStat extends RubyObject {
         return initialize19(fname, unusedBlock);
     }
 
-    @JRubyMethod(name = "initialize", required = 1, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "initialize", visibility = Visibility.PRIVATE)
     public IRubyObject initialize19(IRubyObject fname, Block unusedBlock) {
         Ruby runtime = getRuntime();
         ThreadContext context = runtime.getCurrentContext();
@@ -197,7 +197,7 @@ public class RubyFileStat extends RubyObject {
         return getRuntime().newBoolean(stat.isCharDev());
     }
 
-    @JRubyMethod(name = "<=>", required = 1)
+    @JRubyMethod(name = "<=>")
     public IRubyObject cmp(IRubyObject other) {
         checkInitialized();
         if (!(other instanceof RubyFileStat)) return getRuntime().getNil();
@@ -226,7 +226,11 @@ public class RubyFileStat extends RubyObject {
     @JRubyMethod(name = "birthtime")
     public IRubyObject birthtime() {
         checkInitialized();
-        FileTime btime = null;
+        FileTime btime;
+
+        if (Platform.IS_LINUX || Platform.IS_BSD) {
+            throw getRuntime().newNotImplementedError("birthtime() function is unimplemented on this machine");
+        }
 
         if (file == null || (btime = RubyFile.getBirthtimeWithNIO(file.absolutePath())) == null) {
             return ctime();
@@ -298,7 +302,7 @@ public class RubyFileStat extends RubyObject {
         return getRuntime().newBoolean(stat.isGroupOwned());
     }
     
-    @JRubyMethod(name = "initialize_copy", required = 1, visibility = Visibility.PRIVATE)
+    @JRubyMethod(name = "initialize_copy", visibility = Visibility.PRIVATE)
     @Override
     public IRubyObject initialize_copy(IRubyObject original) {
         if (!(original instanceof RubyFileStat)) {
@@ -346,9 +350,9 @@ public class RubyFileStat extends RubyObject {
                 buf.append("blksize=").append(blockSize(context).inspect().toString()); } catch (Exception e) {} finally { buf.append(", "); }
             try { buf.append("blocks=").append(blocks().inspect().toString()); } catch (Exception e) {} finally { buf.append(", "); }
 
-            buf.append("atime=").append(atime()).append(", ");
-            buf.append("mtime=").append(mtime()).append(", ");
-            buf.append("ctime=").append(ctime());
+            buf.append("atime=").append(atime().inspect()).append(", ");
+            buf.append("mtime=").append(mtime().inspect()).append(", ");
+            buf.append("ctime=").append(ctime().inspect());
             if (Platform.IS_BSD || Platform.IS_MAC) {
                 buf.append(", ").append("birthtime=").append(birthtime());
             }
