@@ -47,6 +47,7 @@ import org.jcodings.specific.USASCIIEncoding;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.ast.util.ArgsUtil;
+import org.jruby.runtime.Arity;
 import org.jruby.runtime.ClassIndex;
 import org.jruby.runtime.Helpers;
 import org.jruby.runtime.JavaSites.FloatSites;
@@ -289,7 +290,7 @@ public class RubyFloat extends RubyNumeric {
     /** flo_coerce
      *
      */
-    @JRubyMethod(name = "coerce", required = 1)
+    @JRubyMethod(name = "coerce")
     @Override
     public IRubyObject coerce(IRubyObject other) {
         final Ruby runtime = metaClass.runtime;
@@ -313,7 +314,7 @@ public class RubyFloat extends RubyNumeric {
     /** flo_plus
      *
      */
-    @JRubyMethod(name = "+", required = 1)
+    @JRubyMethod(name = "+")
     @Override
     public IRubyObject op_plus(ThreadContext context, IRubyObject other) {
         switch (other.getMetaClass().getClassIndex()) {
@@ -332,7 +333,7 @@ public class RubyFloat extends RubyNumeric {
     /** flo_minus
      *
      */
-    @JRubyMethod(name = "-", required = 1)
+    @JRubyMethod(name = "-")
     public IRubyObject op_minus(ThreadContext context, IRubyObject other) {
         switch (other.getMetaClass().getClassIndex()) {
         case INTEGER:
@@ -350,7 +351,7 @@ public class RubyFloat extends RubyNumeric {
     /** flo_mul
      *
      */
-    @JRubyMethod(name = "*", required = 1)
+    @JRubyMethod(name = "*")
     public IRubyObject op_mul(ThreadContext context, IRubyObject other) {
         switch (other.getMetaClass().getClassIndex()) {
         case INTEGER:
@@ -368,7 +369,7 @@ public class RubyFloat extends RubyNumeric {
     /**
      * MRI: flo_div
      */
-    @JRubyMethod(name = "/", required = 1)
+    @JRubyMethod(name = "/")
     public IRubyObject op_div(ThreadContext context, IRubyObject other) { // don't override Numeric#div !
         switch (getMetaClass(other).getClassIndex()) {
         case INTEGER:
@@ -398,7 +399,7 @@ public class RubyFloat extends RubyNumeric {
     /** flo_mod
      *
      */
-    @JRubyMethod(name = {"%", "modulo"}, required = 1)
+    @JRubyMethod(name = {"%", "modulo"})
     public IRubyObject op_mod(ThreadContext context, IRubyObject other) {
         switch (getMetaClass(other).getClassIndex()) {
         case INTEGER:
@@ -431,7 +432,7 @@ public class RubyFloat extends RubyNumeric {
      *
      */
     @Override
-    @JRubyMethod(name = "divmod", required = 1)
+    @JRubyMethod(name = "divmod")
     public IRubyObject divmod(ThreadContext context, IRubyObject other) {
         switch (getMetaClass(other).getClassIndex()) {
         case INTEGER:
@@ -462,7 +463,7 @@ public class RubyFloat extends RubyNumeric {
     /** flo_pow
      *
      */
-    @JRubyMethod(name = "**", required = 1)
+    @JRubyMethod(name = "**")
     public IRubyObject op_pow(ThreadContext context, IRubyObject other) {
         switch (other.getMetaClass().getClassIndex()) {
             case INTEGER:
@@ -490,7 +491,7 @@ public class RubyFloat extends RubyNumeric {
     /** flo_eq
      *
      */
-    @JRubyMethod(name = {"==", "==="}, required = 1)
+    @JRubyMethod(name = {"==", "==="})
     @Override
     public IRubyObject op_equal(ThreadContext context, IRubyObject other) {
         if (Double.isNaN(value)) {
@@ -513,6 +514,13 @@ public class RubyFloat extends RubyNumeric {
         return RubyBoolean.newBoolean(context, value == other);
     }
 
+    public IRubyObject op_not_equal(ThreadContext context, double other) {
+        if (Double.isNaN(value)) {
+            return context.tru;
+        }
+        return RubyBoolean.newBoolean(context, value != other);
+    }
+
     public boolean fastEqual(RubyFloat other) {
         return Double.isNaN(value) ? false : value == other.value;
     }
@@ -532,7 +540,7 @@ public class RubyFloat extends RubyNumeric {
     /** flo_cmp
      *
      */
-    @JRubyMethod(name = "<=>", required = 1)
+    @JRubyMethod(name = "<=>")
     public IRubyObject op_cmp(ThreadContext context, IRubyObject other) {
         final Ruby runtime = context.runtime;
         switch (other.getMetaClass().getClassIndex()) {
@@ -547,16 +555,18 @@ public class RubyFloat extends RubyNumeric {
             FloatSites sites = sites(context);
             if (Double.isInfinite(value) && sites.respond_to_infinite.respondsTo(context, other, other, true)) {
                 IRubyObject infinite = sites.infinite.call(context, other, other);
-                if (infinite.isNil()) {
-                    return value > 0.0 ? RubyFixnum.one(runtime) : RubyFixnum.minus_one(runtime);
-                }
-                long sign = RubyFixnum.fix2long(infinite);
+                if (infinite.isTrue()) {
+                    int sign = RubyComparable.cmpint(context, infinite, this, other);
+
+                    if (sign > 0) {
+                        return value > 0.0 ? RubyFixnum.zero(runtime) : RubyFixnum.minus_one(runtime);
+                    } else {
+                        return value < 0.0 ? RubyFixnum.zero(runtime) : RubyFixnum.one(runtime);
+                    }
 
-                if (sign > 0) {
-                    return value > 0.0 ? RubyFixnum.zero(runtime) : RubyFixnum.minus_one(runtime);
-                } else {
-                    return value < 0.0 ? RubyFixnum.zero(runtime) : RubyFixnum.one(runtime);
                 }
+
+                return value > 0.0 ? RubyFixnum.one(runtime) : RubyFixnum.minus_one(runtime);
             }
             return coerceCmp(context, sites.op_cmp, other);
         }
@@ -569,7 +579,7 @@ public class RubyFloat extends RubyNumeric {
     /** flo_gt
      *
      */
-    @JRubyMethod(name = ">", required = 1)
+    @JRubyMethod(name = ">")
     public IRubyObject op_gt(ThreadContext context, IRubyObject other) {
         switch (other.getMetaClass().getClassIndex()) {
         case INTEGER:
@@ -588,7 +598,7 @@ public class RubyFloat extends RubyNumeric {
     /** flo_ge
      *
      */
-    @JRubyMethod(name = ">=", required = 1)
+    @JRubyMethod(name = ">=")
     public IRubyObject op_ge(ThreadContext context, IRubyObject other) {
         switch (other.getMetaClass().getClassIndex()) {
         case INTEGER:
@@ -607,7 +617,7 @@ public class RubyFloat extends RubyNumeric {
     /** flo_lt
      *
      */
-    @JRubyMethod(name = "<", required = 1)
+    @JRubyMethod(name = "<")
     public IRubyObject op_lt(ThreadContext context, IRubyObject other) {
         switch (other.getMetaClass().getClassIndex()) {
         case INTEGER:
@@ -626,7 +636,7 @@ public class RubyFloat extends RubyNumeric {
     /** flo_le
      *
      */
-    @JRubyMethod(name = "<=", required = 1)
+    @JRubyMethod(name = "<=")
     public IRubyObject op_le(ThreadContext context, IRubyObject other) {
         switch (other.getMetaClass().getClassIndex()) {
         case INTEGER:
@@ -645,7 +655,7 @@ public class RubyFloat extends RubyNumeric {
     /** flo_eql
      *
      */
-    @JRubyMethod(name = "eql?", required = 1)
+    @JRubyMethod(name = "eql?")
     @Override
     public IRubyObject eql_p(IRubyObject other) {
         return metaClass.runtime.newBoolean( equals(other) );
@@ -802,8 +812,10 @@ public class RubyFloat extends RubyNumeric {
     /** float_rationalize
      *
      */
-    @JRubyMethod(name = "rationalize", optional = 1)
+    @JRubyMethod(name = "rationalize", optional = 1, checkArity = false)
     public IRubyObject rationalize(ThreadContext context, IRubyObject[] args) {
+        Arity.checkArgumentCount(context, args, 0, 1);
+
         if (f_negative_p(context, this)) {
             return f_negate(context, f_abs(context, this).rationalize(context, args));
         }
@@ -861,7 +873,7 @@ public class RubyFloat extends RubyNumeric {
      */
     @JRubyMethod(name = "floor")
     public IRubyObject floor(ThreadContext context, IRubyObject digits) {
-        double number, f;
+        double number, f, mul, res;
         int ndigits = num2int(digits);
 
         Ruby runtime = context.runtime;
@@ -879,8 +891,10 @@ public class RubyFloat extends RubyNumeric {
             if (number > 0.0 && floatRoundUnderflow(ndigits, binexp))
                 return newFloat(runtime, 0.0);
             f = Math.pow(10, ndigits);
-            f = Math.floor(number * f) / f;
-            return dbl2num(runtime, f);
+            mul = Math.floor(number * f);
+            res = (mul + 1) / f;
+            if (res > number) res = mul / f;
+            return dbl2num(runtime, res);
         } else {
             RubyInteger num = dbl2ival(runtime, Math.floor(number));
             if (ndigits < 0) num = (RubyInteger) num.floor(context, digits);
@@ -1027,6 +1041,10 @@ public class RubyFloat extends RubyNumeric {
             frexp(number, binexp);
             if (floatRoundOverflow(ndigits, binexp)) return this;
             if (floatRoundUnderflow(ndigits, binexp)) return newFloat(runtime, 0);
+            if (ndigits > 14) {
+                /* In this case, pow(10, ndigits) may not be accurate. */
+                return floatRoundByRational(context, RubyFixnum.newFixnum(runtime, ndigits), mode);
+            }
             f = Math.pow(10, ndigits);
             x = doRound(context, mode, number, f);
             return newFloat(runtime, x / f);
@@ -1103,31 +1121,49 @@ public class RubyFloat extends RubyNumeric {
     }
 
     private static double roundHalfEven(double x, double s) {
-        double f, d, xs = x * s;
+        double u, v, us = 0.0, vs, f, d, uf;
 
         if (x > 0.0) {
-            f = Math.floor(xs);
-            d = xs - f;
+            u = Math.floor(x);
+            v = x - u;
+            us = u * s;
+            vs = v * s;
+            f = Math.floor(vs);
+            uf = us + f;
+            d = vs - f;
             if (d > 0.5)
                 d = 1.0;
-            else if (d == 0.5 || ((double)((f + 0.5) / s) <= x))
-                d = f % 2.0;
+            else if (d == 0.5 || ((double)((uf + 0.5) / s) <= x))
+                d = uf % 2.0;
             else
                 d = 0.0;
             x = f + d;
         }
         else if (x < 0.0) {
-            f = Math.ceil(xs);
-            d = f - xs;
+            u = Math.ceil(x);
+            v = x - u;
+            us = u * s;
+            vs = v * s;
+            f = Math.ceil(vs);
+            uf = us + f;
+            d = f - vs;
             if (d > 0.5)
                 d = 1.0;
-            else if (d == 0.5 || ((double)((f - 0.5) / s) >= x))
-                d = -f % 2.0;
+            else if (d == 0.5 || ((double)((uf - 0.5) / s) >= x))
+                d = -uf % 2.0;
             else
                 d = 0.0;
             x = f - d;
         }
-        return x;
+        return us + x;
+    }
+
+    /** rb_flo_round_by_rational
+     *
+     */
+    private IRubyObject floatRoundByRational(ThreadContext context, IRubyObject ndigits, RoundingMode mode) {
+        IRubyObject v = ((RubyRational)to_r(context)).roundCommon(context, ndigits, mode);
+        return ((RubyRational)v).to_f(context);
     }
 
     /** flo_is_nan_p
@@ -1186,19 +1222,16 @@ public class RubyFloat extends RubyNumeric {
 
     public static RubyFloat unmarshalFrom(UnmarshalStream input) throws java.io.IOException {
         ByteList value = input.unmarshalString();
-        RubyFloat result;
+
         if (value.equals(NAN_BYTELIST)) {
-            result = RubyFloat.newFloat(input.getRuntime(), RubyFloat.NAN);
+            return RubyFloat.newFloat(input.getRuntime(), RubyFloat.NAN);
         } else if (value.equals(NEGATIVE_INFINITY_BYTELIST)) {
-            result = RubyFloat.newFloat(input.getRuntime(), Double.NEGATIVE_INFINITY);
+            return RubyFloat.newFloat(input.getRuntime(), Double.NEGATIVE_INFINITY);
         } else if (value.equals(INFINITY_BYTELIST)) {
-            result = RubyFloat.newFloat(input.getRuntime(), Double.POSITIVE_INFINITY);
+            return RubyFloat.newFloat(input.getRuntime(), Double.POSITIVE_INFINITY);
         } else {
-            result = RubyFloat.newFloat(input.getRuntime(),
-                    ConvertDouble.byteListToDouble19(value, false));
+            return RubyFloat.newFloat(input.getRuntime(), ConvertDouble.byteListToDouble19(value, false));
         }
-        input.registerLinkTarget(result);
-        return result;
     }
 
     private static final ByteList NAN_BYTELIST = new ByteList("nan".getBytes());
