@@ -49,7 +49,6 @@ import java.util.Set;
 import org.jruby.anno.JRubyClass;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.ClassIndex;
-import org.jruby.runtime.JavaSites;
 import org.jruby.runtime.JavaSites.ObjectSites;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.ThreadContext;
@@ -98,7 +97,7 @@ public class RubyObject extends RubyBasicObject {
     }
 
     /**
-     * Path for objects that don't taint and don't enter objectspace.
+     * Path for objects that don't enter objectspace.
      */
     public RubyObject(RubyClass metaClass) {
         super(metaClass);
@@ -400,9 +399,19 @@ public class RubyObject extends RubyBasicObject {
 
     private static boolean fastNumEqualInternal(final ThreadContext context, final IRubyObject a, final IRubyObject b) {
         if (a instanceof RubyFixnum) {
-            if (b instanceof RubyFixnum) return ((RubyFixnum) a).fastEqual((RubyFixnum) b);
+            if (b instanceof RubyFixnum) {
+                if (!context.sites.Fixnum.op_eqq.isBuiltin(a)) {
+                    return context.sites.Fixnum.op_eqq.call(context, a, a, b).isTrue();
+                }
+                return ((RubyFixnum) a).fastEqual((RubyFixnum) b);
+            }
         } else if (a instanceof RubyFloat) {
-            if (b instanceof RubyFloat) return ((RubyFloat) a).fastEqual((RubyFloat) b);
+            if (b instanceof RubyFloat) {
+                if (!context.sites.Float.op_equal.isBuiltin(a)) {
+                    return context.sites.Float.op_equal.call(context, a, a, b).isTrue();
+                }
+                return ((RubyFloat) a).fastEqual((RubyFloat) b);
+            }
         }
         return invokedynamic(context, a, OP_EQUAL, b).isTrue();
     }
