@@ -35,29 +35,38 @@ import jnr.constants.platform.RLIM;
 import jnr.constants.platform.RLIMIT;
 import jnr.constants.platform.Sysconf;
 import jnr.ffi.byref.IntByReference;
+import jnr.posix.Group;
+import jnr.posix.Passwd;
+import jnr.posix.POSIX;
 import jnr.posix.RLimit;
 import jnr.posix.Times;
 import jnr.posix.Timeval;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.anno.JRubyModule;
-import jnr.posix.POSIX;
 import org.jruby.javasupport.Java;
 import org.jruby.javasupport.JavaUtil;
 import org.jruby.platform.Platform;
+import org.jruby.runtime.Arity;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.BlockCallback;
 import org.jruby.runtime.CallBlock;
 import org.jruby.runtime.ObjectAllocator;
+import org.jruby.runtime.ObjectMarshal;
 import org.jruby.runtime.Signature;
 import org.jruby.runtime.ThreadContext;
 
+import static org.jruby.RubyNumeric.fix2long;
 import static org.jruby.runtime.Helpers.throwException;
 import static org.jruby.runtime.Helpers.tryThrow;
 import static org.jruby.runtime.Visibility.*;
 import org.jruby.runtime.builtin.IRubyObject;
+import org.jruby.runtime.builtin.Variable;
+import org.jruby.runtime.component.VariableEntry;
 import org.jruby.runtime.invokedynamic.MethodNames;
 import org.jruby.runtime.marshal.CoreObjectType;
+import org.jruby.runtime.marshal.MarshalStream;
+import org.jruby.runtime.marshal.UnmarshalStream;
 import org.jruby.util.ShellLauncher;
 import org.jruby.exceptions.RaiseException;
 import org.jruby.util.TypeConverter;
@@ -69,11 +78,13 @@ import static org.jruby.runtime.Helpers.invokedynamic;
 import static org.jruby.util.WindowsFFI.kernel32;
 import static org.jruby.util.WindowsFFI.Kernel32.*;
 
+import java.io.IOException;
 import java.lang.invoke.MethodHandle;
 import java.lang.invoke.MethodHandles;
 import java.lang.management.ThreadMXBean;
 import java.lang.management.ManagementFactory;
 import java.lang.reflect.Method;
+import java.util.List;
 import java.util.function.ToIntFunction;
 
 /**
@@ -85,9 +96,9 @@ public class RubyProcess {
     public static RubyModule createProcessModule(Ruby runtime) {
         RubyModule process = runtime.defineModule("Process");
 
-        // TODO: NOT_ALLOCATABLE_ALLOCATOR is probably ok here. Confirm. JRUBY-415
-        RubyClass process_status = process.defineClassUnder("Status", runtime.getObject(), ObjectAllocator.NOT_ALLOCATABLE_ALLOCATOR);
+        RubyClass process_status = process.defineClassUnder("Status", runtime.getObject(), RubyProcess::newAllocatedProcessStatus);
         runtime.setProcStatus(process_status);
+        process_status.setMarshal(PROCESS_STATUS_MARSHAL);
 
         RubyModule process_uid = process.defineModuleUnder("UID");
         runtime.setProcUID(process_uid);
@@ -153,10 +164,47 @@ public class RubyProcess {
     public static final String CLOCK_UNIT_FLOAT_SECOND = "float_second";
     public static final String CLOCK_UNIT_HERTZ = "hertz";
 
+    private static final long PROCESS_STATUS_UNINITIALIZED = -1;
+
+    private static final ObjectMarshal PROCESS_STATUS_MARSHAL = new ObjectMarshal() {
+        @Override
+        public void marshalTo(Ruby runtime, Object obj, RubyClass type,
+                              MarshalStream marshalStream) throws IOException {
+            RubyStatus status = (RubyStatus) obj;
+
+            marshalStream.registerLinkTarget(status);
+            List<Variable<Object>> attrs = status.getMarshalVariableList();
+
+            attrs.add(new VariableEntry("status", runtime.newFixnum(status.status)));
+            attrs.add(new VariableEntry("pid", runtime.newFixnum(status.pid)));
+
+            marshalStream.dumpVariables(attrs);
+        }
+
+        @Override
+        public Object unmarshalFrom(Ruby runtime, RubyClass type, UnmarshalStream input) throws IOException {
+            RubyStatus status = (RubyStatus) input.entry(type.allocate());
+
+            input.ivar(null, status, null);
+
+            RubyFixnum pstatus = (RubyFixnum) status.removeInternalVariable("status");
+            RubyFixnum pid = (RubyFixnum) status.removeInternalVariable("pid");
+
+            status.status = pstatus.getLongValue();
+            status.pid = pid.getLongValue();
+
+            return status;
+        }
+    };
+
+    public static RubyStatus newAllocatedProcessStatus(Ruby runtime, RubyClass metaClass) {
+        return new RubyStatus(runtime, metaClass, 0, PROCESS_STATUS_UNINITIALIZED);
+    }
+
     @JRubyClass(name="Process::Status")
     public static class RubyStatus extends RubyObject {
-        private final long status;
-        private final long pid;
+        private long status;
+        private long pid; // pid or -1 to indicate status was allocate()d and is uninitalized.
 
         private static final long EXIT_SUCCESS = 0L;
         public RubyStatus(Ruby runtime, RubyClass metaClass, long status, long pid) {
@@ -169,6 +217,17 @@ public class RubyProcess {
             return new RubyStatus(runtime, runtime.getProcStatus(), status, pid);
         }
 
+        @JRubyMethod(module = true, optional = 2, checkArity = false)
+        public static IRubyObject wait(ThreadContext context, IRubyObject self, IRubyObject[] args) {
+            int argc = Arity.checkArgumentCount(context, args, 0, 2);
+
+            long pid = argc > 0 ? args[0].convertToInteger().getLongValue() : -1;
+            int flags = argc > 1 ? (int) args[1].convertToInteger().getLongValue() : 0;
+
+            return waitpidStatus(context, pid, flags);
+            //checkErrno(runtime, pid, ECHILD);
+        }
+
         @JRubyMethod(name = "&")
         public IRubyObject op_and(IRubyObject arg) {
             return getRuntime().newFixnum(status & arg.convertToInteger().getLongValue());
@@ -179,17 +238,17 @@ public class RubyProcess {
             return RubyBoolean.newBoolean(getRuntime(), PosixShim.WAIT_MACROS.WIFSTOPPED(status));
         }
 
-        @JRubyMethod(name = {"signaled?"})
+        @JRubyMethod(name = "signaled?")
         public IRubyObject signaled() {
             return RubyBoolean.newBoolean(getRuntime(), PosixShim.WAIT_MACROS.WIFSIGNALED(status));
         }
 
-        @JRubyMethod(name = {"exited?"})
+        @JRubyMethod(name = "exited?")
         public IRubyObject exited() {
             return RubyBoolean.newBoolean(getRuntime(), PosixShim.WAIT_MACROS.WIFEXITED(status));
         }
 
-        @JRubyMethod(name = {"stopsig"})
+        @JRubyMethod
         public IRubyObject stopsig() {
             if (PosixShim.WAIT_MACROS.WIFSTOPPED(status)) {
                 return RubyFixnum.newFixnum(getRuntime(), PosixShim.WAIT_MACROS.WSTOPSIG(status));
@@ -197,7 +256,7 @@ public class RubyProcess {
             return getRuntime().getNil();
         }
 
-        @JRubyMethod(name = {"termsig"})
+        @JRubyMethod
         public IRubyObject termsig() {
             if (PosixShim.WAIT_MACROS.WIFSIGNALED(status)) {
                 return RubyFixnum.newFixnum(getRuntime(), PosixShim.WAIT_MACROS.WTERMSIG(status));
@@ -227,7 +286,7 @@ public class RubyProcess {
             return invokedynamic(context, runtime.newFixnum(status), MethodNames.OP_EQUAL, other);
         }
 
-        @JRubyMethod(name = {"to_i"})
+        @JRubyMethod
         public IRubyObject to_i(ThreadContext context) {
             return to_i(context.runtime);
         }
@@ -250,7 +309,7 @@ public class RubyProcess {
             return RubyBoolean.newBoolean(context, PosixShim.WAIT_MACROS.WEXITSTATUS(status) == EXIT_SUCCESS);
         }
 
-        @JRubyMethod(name = {"coredump?"})
+        @JRubyMethod(name = "coredump?")
         public IRubyObject coredump_p() {
             return RubyBoolean.newBoolean(getRuntime(), PosixShim.WAIT_MACROS.WCOREDUMP(status));
         }
@@ -282,8 +341,16 @@ public class RubyProcess {
             return to_s(getRuntime());
         }
 
+        boolean unitialized() {
+            return pid == PROCESS_STATUS_UNINITIALIZED;
+        }
+
         public IRubyObject inspect(Ruby runtime) {
-            return runtime.newString(pst_message("#<" + getMetaClass().getName() + ": ", pid, status) + ">");
+            if (unitialized()) {
+                return runtime.newString("#<" + getMetaClass().getName() + ": uninitialized>");
+            } else {
+                return runtime.newString(pst_message("#<" + getMetaClass().getName() + ": ", pid, status) + ">");
+            }
         }
 
         // MRI: pst_message
@@ -600,24 +667,28 @@ public class RubyProcess {
         }
     }
 
-    @JRubyMethod(name = "abort", optional = 1, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "abort", optional = 1, checkArity = false, module = true, visibility = PRIVATE)
     public static IRubyObject abort(ThreadContext context, IRubyObject recv, IRubyObject[] args) {
         return RubyKernel.abort(context, recv, args);
     }
 
-    @JRubyMethod(name = "exit!", optional = 1, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "exit!", optional = 1, checkArity = false, module = true, visibility = PRIVATE)
     public static IRubyObject exit_bang(IRubyObject recv, IRubyObject[] args) {
         return RubyKernel.exit_bang(recv, args);
     }
 
     @JRubyMethod(name = "groups", module = true, visibility = PRIVATE)
     public static IRubyObject groups(IRubyObject recv) {
-        long[] groups = Platform.getPlatform().getGroups(recv);
-        RubyArray ary = RubyArray.newArray(recv.getRuntime(), groups.length);
-        for(int i = 0; i < groups.length; i++) {
-            ary.push(RubyFixnum.newFixnum(recv.getRuntime(), groups[i]));
+        final Ruby runtime = recv.getRuntime();
+        long[] groups = runtime.getPosix().getgroups();
+        if (groups == null) { // not-implemented for the given platform (e.g. Windows)
+            throw runtime.newNotImplementedError("groups() function is unimplemented on this machine");
         }
-        return ary;
+        IRubyObject[] ary = new IRubyObject[groups.length];
+        for(int i = 0; i < groups.length; i++) {
+            ary[i] = RubyFixnum.newFixnum(runtime, groups[i]);
+        }
+        return RubyArray.newArrayNoCopy(runtime, ary);
     }
 
     @JRubyMethod(name = "last_status", module = true, visibility = PRIVATE)
@@ -890,7 +961,7 @@ public class RubyProcess {
         return runtime.newFixnum(runtime.getPosix().getpgrp());
     }
 
-    @JRubyMethod(name = "groups=", required = 1, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "groups=", module = true, visibility = PRIVATE)
     public static IRubyObject groups_set(IRubyObject recv, IRubyObject arg) {
         throw recv.getRuntime().newNotImplementedError("Process#groups not yet implemented");
     }
@@ -925,6 +996,18 @@ public class RubyProcess {
         return runtime.newFixnum(pid);
     }
 
+    static IRubyObject waitpidStatus(ThreadContext context, long pid, int flags) {
+        Ruby runtime = context.runtime;
+        int[] status = new int[1];
+        POSIX posix = runtime.getPosix();
+
+        posix.errno(0);
+
+        int res = pthreadKillable(context, ctx -> posix.waitpid(pid, status, flags));
+
+        return RubyProcess.RubyStatus.newProcessStatus(runtime, status[0], res);
+    }
+
     // MRI: rb_waitpid
     public static long waitpid(Ruby runtime, long pid, int flags) {
         int[] status = new int[1];
@@ -1101,12 +1184,22 @@ public class RubyProcess {
     public static IRubyObject egid_set(IRubyObject recv, IRubyObject arg) {
         return egid_set(recv.getRuntime(), arg);
     }
-    @JRubyMethod(name = "egid=", required = 1, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "egid=", module = true, visibility = PRIVATE)
     public static IRubyObject egid_set(ThreadContext context, IRubyObject recv, IRubyObject arg) {
         return egid_set(context.runtime, arg);
     }
     public static IRubyObject egid_set(Ruby runtime, IRubyObject arg) {
-        checkErrno(runtime, runtime.getPosix().setegid((int)arg.convertToInteger().getLongValue()));
+        int gid;
+        if (arg instanceof RubyInteger || arg.checkStringType().isNil()) {
+            gid = (int)arg.convertToInteger().getLongValue();
+        } else {
+            Group group = runtime.getPosix().getgrnam(arg.asJavaString());
+            if (group == null) {
+                throw runtime.newArgumentError("can't find group for " + arg.inspect());
+            }
+            gid = (int)group.getGID();
+        }
+        checkErrno(runtime, runtime.getPosix().setegid(gid));
         return RubyFixnum.zero(runtime);
     }
 
@@ -1126,7 +1219,7 @@ public class RubyProcess {
     public static IRubyObject uid_set(IRubyObject recv, IRubyObject arg) {
         return uid_set(recv.getRuntime(), arg);
     }
-    @JRubyMethod(name = "uid=", required = 1, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "uid=", module = true, visibility = PRIVATE)
     public static IRubyObject uid_set(ThreadContext context, IRubyObject recv, IRubyObject arg) {
         return uid_set(context.runtime, arg);
     }
@@ -1160,7 +1253,7 @@ public class RubyProcess {
     public static IRubyObject getpriority(IRubyObject recv, IRubyObject arg1, IRubyObject arg2) {
         return getpriority(recv.getRuntime(), arg1, arg2);
     }
-    @JRubyMethod(name = "getpriority", required = 2, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "getpriority", module = true, visibility = PRIVATE)
     public static IRubyObject getpriority(ThreadContext context, IRubyObject recv, IRubyObject arg1, IRubyObject arg2) {
         return getpriority(context.runtime, arg1, arg2);
     }
@@ -1197,12 +1290,12 @@ public class RubyProcess {
         return waitpid2(runtime.getCurrentContext(), runtime.getProcess(), args);
     }
 
-    @JRubyMethod(name = "initgroups", required = 2, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "initgroups", module = true, visibility = PRIVATE)
     public static IRubyObject initgroups(IRubyObject recv, IRubyObject arg1, IRubyObject arg2) {
         throw recv.getRuntime().newNotImplementedError("Process#initgroups not yet implemented");
     }
 
-    @JRubyMethod(name = "maxgroups=", required = 1, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "maxgroups=", module = true, visibility = PRIVATE)
     public static IRubyObject maxgroups_set(IRubyObject recv, IRubyObject arg) {
         throw recv.getRuntime().newNotImplementedError("Process#maxgroups_set not yet implemented");
     }
@@ -1225,7 +1318,7 @@ public class RubyProcess {
     public static IRubyObject gid_set(IRubyObject recv, IRubyObject arg) {
         return gid_set(recv.getRuntime(), arg);
     }
-    @JRubyMethod(name = "gid=", required = 1, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "gid=", module = true, visibility = PRIVATE)
     public static IRubyObject gid_set(ThreadContext context, IRubyObject recv, IRubyObject arg) {
         return gid_set(context.runtime, arg);
     }
@@ -1248,12 +1341,22 @@ public class RubyProcess {
     public static IRubyObject euid_set(IRubyObject recv, IRubyObject arg) {
         return euid_set(recv.getRuntime(), arg);
     }
-    @JRubyMethod(name = "euid=", required = 1, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "euid=", module = true, visibility = PRIVATE)
     public static IRubyObject euid_set(ThreadContext context, IRubyObject recv, IRubyObject arg) {
         return euid_set(context.runtime, arg);
     }
     public static IRubyObject euid_set(Ruby runtime, IRubyObject arg) {
-        checkErrno(runtime, runtime.getPosix().seteuid((int)arg.convertToInteger().getLongValue()));
+        int uid;
+        if (arg instanceof RubyInteger || arg.checkStringType().isNil()) {
+            uid = (int)arg.convertToInteger().getLongValue();
+        } else {
+            Passwd password = runtime.getPosix().getpwnam(arg.asJavaString());
+            if (password == null) {
+                throw runtime.newArgumentError("can't find user for " + arg.inspect());
+            }
+            uid = (int)password.getUID();
+        }
+        checkErrno(runtime, runtime.getPosix().seteuid(uid));
         return RubyFixnum.zero(runtime);
     }
 
@@ -1261,7 +1364,7 @@ public class RubyProcess {
     public static IRubyObject setpriority(IRubyObject recv, IRubyObject arg1, IRubyObject arg2, IRubyObject arg3) {
         return setpriority(recv.getRuntime(), arg1, arg2, arg3);
     }
-    @JRubyMethod(name = "setpriority", required = 3, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "setpriority", module = true, visibility = PRIVATE)
     public static IRubyObject setpriority(ThreadContext context, IRubyObject recv, IRubyObject arg1, IRubyObject arg2, IRubyObject arg3) {
         return setpriority(context.runtime, arg1, arg2, arg3);
     }
@@ -1279,7 +1382,7 @@ public class RubyProcess {
     public static IRubyObject setpgid(IRubyObject recv, IRubyObject arg1, IRubyObject arg2) {
         return setpgid(recv.getRuntime(), arg1, arg2);
     }
-    @JRubyMethod(name = "setpgid", required = 2, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "setpgid", module = true, visibility = PRIVATE)
     public static IRubyObject setpgid(ThreadContext context, IRubyObject recv, IRubyObject arg1, IRubyObject arg2) {
         return setpgid(context.runtime, arg1, arg2);
     }
@@ -1293,7 +1396,7 @@ public class RubyProcess {
     public static IRubyObject getpgid(IRubyObject recv, IRubyObject arg) {
         return getpgid(recv.getRuntime(), arg);
     }
-    @JRubyMethod(name = "getpgid", required = 1, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "getpgid", module = true, visibility = PRIVATE)
     public static IRubyObject getpgid(ThreadContext context, IRubyObject recv, IRubyObject arg) {
         return getpgid(context.runtime, arg);
     }
@@ -1305,7 +1408,7 @@ public class RubyProcess {
     public static IRubyObject getrlimit(IRubyObject recv, IRubyObject arg) {
         return getrlimit(recv.getRuntime(), arg);
     }
-    @JRubyMethod(name = "getrlimit", required = 1, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "getrlimit", module = true, visibility = PRIVATE)
     public static IRubyObject getrlimit(ThreadContext context, IRubyObject recv, IRubyObject arg) {
         return getrlimit(context.runtime, arg);
     }
@@ -1460,7 +1563,7 @@ public class RubyProcess {
 
     }
 
-    @JRubyMethod(name = "detach", required = 1, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "detach", module = true, visibility = PRIVATE)
     public static IRubyObject detach(ThreadContext context, IRubyObject recv, IRubyObject arg) {
         final long pid = arg.convertToInteger().getLongValue();
         Ruby runtime = context.runtime;
@@ -1658,6 +1761,11 @@ public class RubyProcess {
         return RubyKernel.fork(context, recv, block);
     }
 
+    @JRubyMethod(module = true, visibility = PRIVATE, notImplemented = true)
+    public static IRubyObject _fork(ThreadContext context, IRubyObject recv, Block block) {
+        throw context.runtime.newNotImplementedError("fork is not available on this platform");
+    }
+
     @JRubyMethod(name = "fork", module = true, visibility = PRIVATE, notImplemented = true)
     public static IRubyObject fork19(ThreadContext context, IRubyObject recv, Block block) {
         return RubyKernel.fork(context, recv, block);
@@ -1674,7 +1782,7 @@ public class RubyProcess {
         return RubyFixnum.newFixnum(runtime, ShellLauncher.runExternalWithoutWait(runtime, args));
     }
 
-    @JRubyMethod(name = "exit", optional = 1, module = true, visibility = PRIVATE)
+    @JRubyMethod(name = "exit", optional = 1, checkArity = false, module = true, visibility = PRIVATE)
     public static IRubyObject exit(IRubyObject recv, IRubyObject[] args) {
         return RubyKernel.exit(recv, args);
     }
