@@ -68,6 +68,7 @@ import org.jruby.util.IdUtil;
 import org.jruby.util.PerlHash;
 import org.jruby.util.SipHashInline;
 import org.jruby.util.SymbolNameType;
+import org.jruby.util.TypeConverter;
 
 import java.lang.ref.WeakReference;
 import java.util.concurrent.locks.ReentrantLock;
@@ -92,6 +93,7 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
     private final ByteList symbolBytes;
     private final int hashCode;
     private String decodedString;
+    private RubyString rubyString;
     private transient Object constant;
     private SymbolNameType type;
 
@@ -180,7 +182,7 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
     public final String toString() {
         String decoded = decodedString;
         if (decoded == null) {
-            decodedString = decoded = RubyEncoding.decodeRaw(symbolBytes);
+            decodedString = decoded = RubyEncoding.decodeRaw(getBytes());
         }
         return decoded;
     }
@@ -283,7 +285,7 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
 
 
     public boolean validLocalVariableName() {
-        return type == SymbolNameType.LOCAL;
+        return type == SymbolNameType.LOCAL && !"nil".equals(idString());
     }
 
     @Override
@@ -443,9 +445,10 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
         Encoding resenc = runtime.getDefaultInternalEncoding();
         if (resenc == null) resenc = runtime.getDefaultExternalEncoding();
 
-        RubyString str = RubyString.newString(runtime, symbolBytes);
+        RubyString str = RubyString.newString(runtime, getBytes());
 
-        if (!(isPrintable(runtime) && (resenc.equals(symbolBytes.getEncoding()) || str.isAsciiOnly()) && isSymbolName(symbol))) {
+        if (!(isPrintable(runtime) && (resenc.equals(getBytes().getEncoding()) || str.isAsciiOnly()) &&
+                isSymbolName(symbol))) {
             str = str.inspect(runtime);
         }
 
@@ -467,13 +470,24 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
         return to_s(metaClass.runtime);
     }
 
-    @JRubyMethod
+    @JRubyMethod(name = "to_s")
     public IRubyObject to_s(ThreadContext context) {
         return to_s(context.runtime);
     }
 
     final RubyString to_s(Ruby runtime) {
-        return RubyString.newStringShared(runtime, symbolBytes);
+        return RubyString.newStringShared(runtime, getBytes());
+    }
+
+    @JRubyMethod(name = "name")
+    public IRubyObject name(ThreadContext context) {
+      // FIXME: A race here where two in-flight to_s on same symbol can return a different instance.
+      if (rubyString == null) {
+        rubyString = RubyString.newStringShared(context.runtime, getBytes());
+        rubyString.setFrozen(true);
+      }
+
+      return rubyString;
     }
 
     public IRubyObject id2name() {
@@ -490,13 +504,13 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
         return to_s(metaClass.runtime);
     }
 
-    @JRubyMethod(name = "===", required = 1)
+    @JRubyMethod(name = "===")
     @Override
     public IRubyObject op_eqq(ThreadContext context, IRubyObject other) {
         return RubyBoolean.newBoolean(context, this == other);
     }
 
-    @JRubyMethod(name = "==", required = 1)
+    @JRubyMethod(name = "==")
     @Override
     public IRubyObject op_equal(ThreadContext context, IRubyObject other) {
         return RubyBoolean.newBoolean(context, this == other);
@@ -545,11 +559,6 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
     @Deprecated
     public IRubyObject to_sym19() { return this; }
 
-    @Override
-    public IRubyObject taint(ThreadContext context) {
-        return this;
-    }
-
     private RubyString newShared(Ruby runtime) {
         return RubyString.newStringShared(runtime, symbolBytes);
     }
@@ -601,7 +610,7 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
         return newShared(context.runtime).match19(context, other, pos, block);
     }
 
-    @JRubyMethod(name = "match", required = 1, rest = true)
+    @JRubyMethod(name = "match", required = 1, rest = true, checkArity = false)
     public IRubyObject match_m(ThreadContext context, IRubyObject[] args, Block block) {
         return newShared(context.runtime).match19(context, args, block);
     }
@@ -637,6 +646,46 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
         return newShared(context.runtime).empty_p(context);
     }
 
+    @JRubyMethod(name = "start_with?")
+    public IRubyObject start_with_p(ThreadContext context) {
+        return context.fals;
+    }
+
+    @JRubyMethod(name = "start_with?")
+    public IRubyObject start_with_p(ThreadContext context, IRubyObject arg) {
+        if (arg instanceof RubyRegexp) {
+            return ((RubyRegexp) arg).startsWith(context, to_s(context.runtime)) ? context.tru : context.fals;
+        }
+        return to_s(context.runtime).startsWith(arg.convertToString()) ? context.tru : context.fals;
+    }
+
+    @JRubyMethod(name = "start_with?", rest = true)
+    public IRubyObject start_with_p(ThreadContext context, IRubyObject[] args) {
+        for (int i = 0; i < args.length; i++) {
+            if (start_with_p(context, args[i]).isTrue()) return context.tru;
+        }
+        return context.fals;
+    }
+
+    @JRubyMethod(name = "end_with?")
+    public IRubyObject end_with_p(ThreadContext context) {
+        return context.fals;
+    }
+
+    @JRubyMethod(name = "end_with?")
+    public IRubyObject end_with_p(ThreadContext context, IRubyObject arg) {
+        return to_s(context.runtime).endWith(arg) ? context.tru : context.fals;
+    }
+
+    @JRubyMethod(name = "end_with?", rest = true)
+    public IRubyObject end_with_p(ThreadContext context, IRubyObject[]args) {
+        RubyString str = to_s(context.runtime);
+        for (int i = 0; i < args.length; i++) {
+            if (str.endWith(args[i])) return context.tru;
+        }
+        return context.fals;
+    }
+
     @JRubyMethod
     public IRubyObject upcase(ThreadContext context) {
         Ruby runtime = context.runtime;
@@ -720,7 +769,7 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
 
         return RubyProc.newProc(context.runtime,
                                 new Block(body, Block.NULL_BLOCK.getBinding()),
-                                Block.Type.PROC);
+                                Block.Type.LAMBDA);
     }
 
     public IRubyObject toRefinedProc(ThreadContext context, StaticScope scope) {
@@ -728,7 +777,7 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
 
         return RubyProc.newProc(context.runtime,
                 new Block(body, Block.NULL_BLOCK.getBinding()),
-                Block.Type.PROC);
+                Block.Type.LAMBDA);
     }
 
     private static boolean isIdentStart(char c) {
@@ -777,6 +826,7 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
     }
 
     private boolean isPrintable(final Ruby runtime) {
+        ByteList symbolBytes = getBytes();
         int p = symbolBytes.getBegin();
         int end = p + symbolBytes.getRealSize();
         byte[] bytes = symbolBytes.getUnsafeBytes();
@@ -841,7 +891,7 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
     private static boolean isSymbolLocal(final String str, final char first, final int length) {
         if (!isIdentStart(first)) return false;
 
-        boolean localID = (first >= 'a' && first <= 'z');
+        boolean localID = isIdentStart(first);
         int last = 1;
 
         for (; last < length; last++) {
@@ -884,14 +934,13 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
         // Need symbol to register before encoding, so pass in a lambda for remaining unmarshal logic
         RubySymbol result = newSymbol(input.getRuntime(), byteList,
                 (sym, newSym) -> {
-                    input.registerLinkTarget(sym);
-
                     // get encoding from stream and set into symbol
-                    if (state.isIvarWaiting()) {
+                    if (state != null && state.isIvarWaiting()) {
                         try {
                             input.unmarshalInt(); // throw-away, always single ivar of encoding
+                            IRubyObject value = input.unmarshalObject();
 
-                            Encoding enc = input.getEncodingFromUnmarshaled(input.unmarshalObject());
+                            Encoding enc = input.symbolToEncoding(sym, value);
                             if (enc == null) throw new RuntimeException("BUG: No encoding found in marshal stream");
 
                             // only change encoding if the symbol has been newly-created
@@ -922,13 +971,13 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
 
     @Override
     public Encoding getEncoding() {
-        return symbolBytes.getEncoding();
+        return getBytes().getEncoding();
     }
 
     @Override
     public void setEncoding(Encoding e) {
-        symbolBytes.setEncoding(e);
-        type = IdUtil.determineSymbolNameType(getRuntime(), symbolBytes);
+        getBytes().setEncoding(e);
+        type = IdUtil.determineSymbolNameType(getRuntime(), getBytes());
     }
 
     public static final class SymbolTable {
@@ -1381,6 +1430,11 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
         return h;
     }
 
+    @Override
+    public Class getJavaClass() {
+        return String.class;
+    }
+
     @Override
     public boolean shouldMarshalEncoding() {
         Encoding enc = getMarshalEncoding();
@@ -1390,7 +1444,7 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
 
     @Override
     public Encoding getMarshalEncoding() {
-        return symbolBytes.getEncoding();
+        return getBytes().getEncoding();
     }
 
     /**
@@ -1417,17 +1471,37 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
         return object.convertToString().getByteList().toString();
     }
 
-    private static final class SymbolProcBody extends ContextAwareBlockBody {
-        private final CallSite site;
+    // MRI: rb_check_id
+    public static String checkID(IRubyObject object) {
+        if (object instanceof RubySymbol) return ((RubySymbol) object).idString();
 
-        public SymbolProcBody(Ruby runtime, String symbol) {
-            super(runtime.getStaticScopeFactory().getDummyScope(), Signature.OPTIONAL);
-            this.site = MethodIndex.getFunctionalCallSite(symbol);
+        if (!(object instanceof RubyString)) {
+            IRubyObject tmp = TypeConverter.checkStringType(object.getRuntime(), object);
+
+            if (tmp.isNil()) {
+                throw object.getRuntime().newTypeError(str(object.getRuntime(), "", object, " is not a symbol nor a string"));
+            }
+
+            object = tmp;
         }
 
-        public SymbolProcBody(Ruby runtime, String symbol, StaticScope scope) {
+        return ((RubyString) object).getByteList().toString();
+    }
+
+    public static final class SymbolProcBody extends ContextAwareBlockBody {
+        private final CallSite site;
+        private final String id;
+
+        public SymbolProcBody(Ruby runtime, String id) {
+            super(runtime.getStaticScopeFactory().getDummyScope(), Signature.OPTIONAL);
+            this.site = MethodIndex.getFunctionalCallSite(id);
+            this.id = id;
+        }
+
+        public SymbolProcBody(Ruby runtime, String id, StaticScope scope) {
             super(scope, Signature.OPTIONAL);
-            this.site = new RefinedCachingCallSite(symbol, scope, CallType.FUNCTIONAL);
+            this.site = new RefinedCachingCallSite(id, scope, CallType.FUNCTIONAL);
+            this.id = id;
         }
 
         private IRubyObject yieldInner(ThreadContext context, RubyArray array, Block blockArg) {
@@ -1487,9 +1561,19 @@ public class RubySymbol extends RubyObject implements MarshalEncoding, EncodingC
             return -1;
         }
 
+        public String getId() {
+            return id;
+        }
+
         @Override
         public ArgumentDescriptor[] getArgumentDescriptors() {
-            return ArgumentDescriptor.ANON_REST;
+            return ArgumentDescriptor.SYMBOL_PROC;
         }
     }
+
+    @Deprecated
+    @Override
+    public IRubyObject taint(ThreadContext context) {
+        return this;
+    }
 }
