@@ -28,6 +28,7 @@ import org.jruby.RubyNumeric;
 import org.jruby.RubyString;
 import org.jruby.RubyStruct;
 import org.jruby.ext.rbconfig.RbConfigLibrary;
+import org.jruby.runtime.Arity;
 import org.jruby.runtime.Block;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
@@ -39,7 +40,7 @@ import java.nio.ByteBuffer;
 @JRubyModule(name="Etc")
 public class RubyEtc {
     public static class IOExt {
-        @JRubyMethod(required = 1)
+        @JRubyMethod
         public static synchronized IRubyObject pathconf(ThreadContext context, IRubyObject recv, IRubyObject arg) {
             Pathconf name = Pathconf.valueOf(RubyNumeric.num2long(arg));
             RubyIO io = (RubyIO) recv;
@@ -158,16 +159,20 @@ public class RubyEtc {
         return RubyArray.newArrayMayCopy(runtime, arr);
     }
     
-    @JRubyMethod(required = 1, module = true)
+    @JRubyMethod(module = true)
     public static synchronized IRubyObject sysconf(ThreadContext context, IRubyObject recv, IRubyObject arg) {
         Ruby runtime = context.runtime;
         Sysconf name = Sysconf.valueOf(RubyNumeric.num2long(arg));
         POSIX posix = runtime.getPosix();
+        posix.errno(0);
         long ret = posix.sysconf(name);
+
         if (ret == -1) {
-            if (posix.errno() == Errno.ENOENT.intValue()) {
-                return runtime.getNil();
-            } else if (posix.errno() == Errno.EOPNOTSUPP.intValue()) {
+            int errno = posix.errno();
+
+            if (errno == Errno.ENOENT.intValue() || errno == 0) {
+                return context.nil;
+            } else if (errno == Errno.EOPNOTSUPP.intValue()) {
                 throw runtime.newNotImplementedError("sysconf() function is unimplemented on this machine");
             } else {
                 throw runtime.newErrnoFromLastPOSIXErrno();
@@ -176,7 +181,7 @@ public class RubyEtc {
         return RubyFixnum.newFixnum(runtime, ret);
     }
     
-    @JRubyMethod(required = 1, module = true)
+    @JRubyMethod(module = true)
     public static synchronized IRubyObject confstr(ThreadContext context, IRubyObject recv, IRubyObject arg) {
         Confstr name = Confstr.valueOf(RubyNumeric.num2long(arg));
         ByteBuffer buf;
@@ -207,9 +212,12 @@ public class RubyEtc {
         return RubyString.newString(context.runtime, bytes);
     }
 
-    @JRubyMethod(optional=1, module = true)
+    @JRubyMethod(optional = 1, checkArity = false, module = true)
     public static synchronized IRubyObject getpwuid(IRubyObject recv, IRubyObject[] args) {
         Ruby runtime = recv.getRuntime();
+
+        int argc = Arity.checkArgumentCount(runtime, args, 0, 1);
+
         POSIX posix = runtime.getPosix();
         IRubyObject oldExc = runtime.getGlobalVariables().get("$!"); // Save $!
         try {
@@ -236,7 +244,7 @@ public class RubyEtc {
         }
     }
 
-    @JRubyMethod(required=1, module = true)
+    @JRubyMethod(module = true)
     public static synchronized IRubyObject getpwnam(IRubyObject recv, IRubyObject name) {
         Ruby runtime = recv.getRuntime();
         String nam = name.convertToString().toString();
@@ -365,7 +373,7 @@ public class RubyEtc {
         }
     }
 
-    @JRubyMethod(required=1, module = true)
+    @JRubyMethod(module = true)
     public static synchronized IRubyObject getgrnam(IRubyObject recv, IRubyObject name) {
         Ruby runtime = recv.getRuntime();
         String nam = name.convertToString().toString();
@@ -388,13 +396,16 @@ public class RubyEtc {
         }
     }
 
-    @JRubyMethod(optional=1, module = true)
+    @JRubyMethod(optional = 1, checkArity = false, module = true)
     public static synchronized IRubyObject getgrgid(IRubyObject recv, IRubyObject[] args) {
         Ruby runtime = recv.getRuntime();
+
+        int argc = Arity.checkArgumentCount(runtime, args, 0, 1);
+
         POSIX posix = runtime.getPosix();
 
         try {
-            int gid = args.length == 0 ? posix.getgid() : RubyNumeric.fix2int(args[0]);
+            int gid = argc == 0 ? posix.getgid() : RubyNumeric.fix2int(args[0]);
             Group gr = posix.getgrgid(gid);
             if(gr == null) {
                 if (Platform.IS_WINDOWS) {  // MRI behavior
@@ -518,7 +529,6 @@ public class RubyEtc {
             if (commonAppData != null) tmp = ByteList.create(commonAppData);
         }
         RubyString ret = RubyString.newString(runtime, tmp, runtime.getDefaultExternalEncoding());
-        ret.untaint(context);
 
         return ret;
     }
@@ -534,7 +544,6 @@ public class RubyEtc {
             if (localAppData != null) tmp = ByteList.create(localAppData);
         }
         RubyString ret = RubyString.newString(runtime, tmp, runtime.getDefaultExternalEncoding());
-        ret.untaint(context);
 
         return ret;
     }
