@@ -2,11 +2,16 @@ package org.jruby.ext.fiber;
 
 import org.jruby.Ruby;
 import org.jruby.RubyArray;
+import org.jruby.RubyBasicObject;
 import org.jruby.RubyBoolean;
 import org.jruby.RubyClass;
+import org.jruby.RubyException;
+import org.jruby.RubyFixnum;
+import org.jruby.RubyKernel;
 import org.jruby.RubyObject;
 import org.jruby.RubyThread;
 import org.jruby.anno.JRubyMethod;
+import org.jruby.ast.util.ArgsUtil;
 import org.jruby.exceptions.JumpException;
 import org.jruby.exceptions.RaiseException;
 import org.jruby.ir.operands.IRException;
@@ -19,10 +24,12 @@ import org.jruby.runtime.Helpers;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.Visibility;
 import org.jruby.runtime.builtin.IRubyObject;
+import org.jruby.util.cli.Options;
 import org.jruby.util.log.Logger;
 import org.jruby.util.log.LoggerFactory;
 
 import java.lang.invoke.MethodHandle;
+import java.lang.invoke.MethodHandles;
 import java.lang.ref.WeakReference;
 import java.lang.reflect.Method;
 import java.util.Map;
@@ -34,44 +41,66 @@ public class ThreadFiber extends RubyObject implements ExecutionContext {
     private static final Logger LOG = LoggerFactory.getLogger(ThreadFiber.class);
 
     private static final BiConsumer<Ruby, Runnable> FIBER_LAUNCHER;
+    private static final MethodHandle VTHREAD_START_METHOD;
 
     static {
-        BiConsumer<Ruby, Runnable> fiberLauncher = null;
+        BiConsumer<Ruby, Runnable> fiberLauncher = ThreadFiber::nativeThreadLauncher;
         MethodHandle start = null;
 
-        try {
-            // test that API is available
-            Method ofVirtualMethod = Thread.class.getMethod("ofVirtual");
-            Object builder = ofVirtualMethod.invoke(null);
-            Method startMethod = Class.forName("java.lang.Thread$Builder").getMethod("start", Runnable.class);
+        if (Options.FIBER_VTHREADS.load()) {
+            try {
+                // test that API is available
+                Method ofVirtualMethod = Thread.class.getMethod("ofVirtual");
+                Object builder = ofVirtualMethod.invoke(null);
+                Method startMethod = Class.forName("java.lang.Thread$Builder").getMethod("start", Runnable.class);
 
-            fiberLauncher = (runtime, runnable) -> {
-                try {
-                    startMethod.invoke(builder, runnable);
-                } catch (Throwable t) {
-                    Helpers.throwException(t);
-                }
-            };
-        } catch (Throwable t) {
-            fiberLauncher = (runtime, runnable) -> runtime.getFiberExecutor().submit(runnable);
-        }
+                start = MethodHandles.publicLookup().unreflect(startMethod).bindTo(builder);
 
-        if (fiberLauncher == null) {
+                fiberLauncher = new VirtualThreadLauncher();
+            } catch (Throwable t) {
+                // default impl set below
+            }
         }
 
+        VTHREAD_START_METHOD = start;
         FIBER_LAUNCHER = fiberLauncher;
     }
 
+    private static void nativeThreadLauncher(Ruby runtime, Runnable runnable) {
+        runtime.getFiberExecutor().submit(runnable);
+    }
+
+    public boolean isBlocking() {
+        return data.blocking;
+    }
+
+    private static class VirtualThreadLauncher implements BiConsumer<Ruby, Runnable> {
+        @Override
+        public void accept(Ruby ruby, Runnable runnable) {
+            try {
+                VTHREAD_START_METHOD.invokeWithArguments(runnable);
+            } catch (Throwable t) {
+                Helpers.throwException(t);
+            }
+        }
+    }
+
     public ThreadFiber(Ruby runtime, RubyClass klass) {
+        this(runtime, klass, false);
+    }
+
+    public ThreadFiber(Ruby runtime, RubyClass klass, boolean root) {
         super(runtime, klass);
+
+        this.root = root;
     }
 
     public static void initRootFiber(ThreadContext context, RubyThread currentThread) {
         Ruby runtime = context.runtime;
 
-        ThreadFiber rootFiber = new ThreadFiber(runtime, runtime.getFiber());
+        ThreadFiber rootFiber = new ThreadFiber(runtime, runtime.getFiber(), true);
 
-        rootFiber.data = new FiberData(new FiberQueue(runtime), currentThread, rootFiber);
+        rootFiber.data = new FiberData(new FiberQueue(runtime), currentThread, rootFiber, true);
         rootFiber.thread = currentThread;
         context.setRootFiber(rootFiber);
     }
@@ -79,28 +108,61 @@ public class ThreadFiber extends RubyObject implements ExecutionContext {
     @JRubyMethod(visibility = Visibility.PRIVATE)
     public IRubyObject initialize(ThreadContext context, Block block) {
         Ruby runtime = context.runtime;
-        
+
         if (!block.isGiven()) throw runtime.newArgumentError("tried to create Proc object without block");
 
-        data = new FiberData(new FiberQueue(runtime), context.getFiberCurrentThread(), this);
-        
+        data = new FiberData(new FiberQueue(runtime), context.getFiberCurrentThread(), this, false);
+
         FiberData currentFiberData = context.getFiber().data;
-        
+
         thread = createThread(runtime, data, currentFiberData.queue, block);
-        
+
+        return context.nil;
+    }
+
+    @JRubyMethod(visibility = Visibility.PRIVATE)
+    public IRubyObject initialize(ThreadContext context, IRubyObject _opts, Block block) {
+        Ruby runtime = context.runtime;
+
+        if (!block.isGiven()) throw runtime.newArgumentError("tried to create Proc object without block");
+
+        IRubyObject opts = ArgsUtil.getOptionsArg(runtime, _opts);
+
+        boolean blocking = false;
+
+        if (!opts.isNil()) {
+            IRubyObject[] blockingPoolOpt = ArgsUtil.extractKeywordArgs(context, opts, "blocking", "pool");
+
+            if (blockingPoolOpt != null) {
+                IRubyObject blockingOpt = blockingPoolOpt[0];
+                if (blockingOpt != null && !blockingOpt.isNil()) {
+                    blocking = blockingOpt.isTrue();
+                }
+
+                // TODO: pooling
+            }
+        }
+
+        data = new FiberData(new FiberQueue(runtime), context.getFiberCurrentThread(), this, blocking);
+
+        FiberData currentFiberData = context.getFiber().data;
+
+        thread = createThread(runtime, data, currentFiberData.queue, block);
+
         return context.nil;
     }
     
-    @JRubyMethod(rest = true)
+    @JRubyMethod(rest = true, keywords = true)
     public IRubyObject resume(ThreadContext context, IRubyObject[] values) {
         Ruby runtime = context.runtime;
 
-        final FiberData data = this.data;
-        if (data.prev != null || data.transferred) throw runtime.newFiberError("double resume");
-        
         if (!alive()) throw runtime.newFiberError("dead fiber called");
-        
+
+        final FiberData data = this.data;
         FiberData currentFiberData = context.getFiber().data;
+
+        if (currentFiberData == data) throw runtime.newFiberError("attempt to resume the current fiber");
+        if (root || data.prev != null || data.transferred) throw runtime.newFiberError("attempt to resume a resuming fiber");
         
         if (data == currentFiberData) {
             switch (values.length) {
@@ -110,27 +172,60 @@ public class ThreadFiber extends RubyObject implements ExecutionContext {
             }
         }
         
-        IRubyObject val;
+        FiberRequest val;
         switch (values.length) {
             case 0: val = NEVER; break;
-            case 1: val = values[0]; break;
-            default: val = RubyArray.newArrayMayCopy(runtime, values);
+            case 1: val = new FiberRequest(values[0], RequestType.DATA); break;
+            default: val = new FiberRequest(RubyArray.newArrayMayCopy(runtime, values), RequestType.DATA);
         }
         
         if (data.parent != context.getFiberCurrentThread()) throw runtime.newFiberError("fiber called across threads");
         
         data.prev = context.getFiber();
 
+        FiberRequest result;
         try {
-            return exchangeWithFiber(context, currentFiberData, data, val);
+            result = exchangeWithFiber(context, currentFiberData, data, val);
         } finally {
             data.prev = null;
         }
+
+        if (data.blocking) {
+            context.getFiberCurrentThread().decrementBlocking();
+        }
+
+        if (result.type == RequestType.RAISE) {
+            throw ((RubyException) result.data).toThrowable();
+        }
+
+        return processResultData(context, result);
     }
 
-    private static IRubyObject exchangeWithFiber(ThreadContext context, FiberData currentFiberData, FiberData targetFiberData, IRubyObject val) {
+    @JRubyMethod(rest = true)
+    public IRubyObject raise(ThreadContext context, IRubyObject[] args) {
+        return raise(context, RubyThread.prepareRaiseException(context, args));
+    }
+
+    public enum RequestType {
+        DATA,
+        RAISE
+    }
+
+    public static class FiberRequest {
+        final IRubyObject data;
+        final RequestType type;
+
+        FiberRequest(IRubyObject data, RequestType type) {
+            this.data = data;
+            this.type = type;
+        }
+    }
+
+    private static final FiberRequest NEVER = new FiberRequest(RubyBasicObject.NEVER, RequestType.DATA);
+
+    private static FiberRequest exchangeWithFiber(ThreadContext context, FiberData currentFiberData, FiberData targetFiberData, FiberRequest request) {
         // push should not block and does not check interrupts
-        targetFiberData.queue.push(context, val);
+        targetFiberData.queue.push(context, request);
 
         // At this point we consider ourselves "in" the resume, so we need to make an effort to propagate any interrupt
         // raise to the fiber. We forward the exception using the interrupt mechanism and then try once more to let the
@@ -139,14 +234,28 @@ public class ThreadFiber extends RubyObject implements ExecutionContext {
         // interrupted again and must abandon the fiber.
 
         try {
-            IRubyObject result = currentFiberData.queue.pop(context);
-            return result == NEVER ? context.nil : result;
+            adjustThreadBlocking(context, currentFiberData, targetFiberData);
+
+            return currentFiberData.queue.pop(context);
         } catch (RaiseException re) {
             handleExceptionDuringExchange(context, currentFiberData, targetFiberData, re);
 
             // if we get here, we forwarded exception so try once more
-            IRubyObject result = currentFiberData.queue.pop(context);
-            return result == NEVER ? context.nil : result;
+            return currentFiberData.queue.pop(context);
+        } finally {
+            adjustThreadBlocking(context, targetFiberData, currentFiberData);
+        }
+    }
+
+    private static void adjustThreadBlocking(ThreadContext context, FiberData currentFiberData, FiberData targetFiberData) {
+        // if fiber we are leaving is blocking, decrement thread blocking count
+        if (currentFiberData.blocking) {
+            context.getFiberCurrentThread().decrementBlocking();
+        }
+
+        // if fiber we are entering is blocking, increment thread blocking count
+        if (targetFiberData.blocking) {
+            context.getFiberCurrentThread().incrementBlocking();
         }
     }
 
@@ -196,7 +305,7 @@ public class ThreadFiber extends RubyObject implements ExecutionContext {
     }
 
     @JRubyMethod(rest = true)
-    public IRubyObject __transfer__(ThreadContext context, IRubyObject[] values) {
+    public IRubyObject transfer(ThreadContext context, IRubyObject[] values) {
         Ruby runtime = context.runtime;
 
         final FiberData data = this.data;
@@ -214,11 +323,11 @@ public class ThreadFiber extends RubyObject implements ExecutionContext {
             }
         }
         
-        IRubyObject val;
+        FiberRequest val;
         switch (values.length) {
             case 0: val = NEVER; break;
-            case 1: val = values[0]; break;
-            default: val = RubyArray.newArrayMayCopy(runtime, values);
+            case 1: val = new FiberRequest(values[0], RequestType.DATA); break;
+            default: val = new FiberRequest(RubyArray.newArrayMayCopy(runtime, values), RequestType.DATA);
         }
         
         if (data.parent != context.getFiberCurrentThread()) throw runtime.newFiberError("fiber called across threads");
@@ -231,13 +340,57 @@ public class ThreadFiber extends RubyObject implements ExecutionContext {
         } else {
             data.prev = context.getFiber();
         }
-        
+
+        FiberRequest result;
         try {
-            return exchangeWithFiber(context, currentFiberData, data, val);
+            result = exchangeWithFiber(context, currentFiberData, data, val);
         } finally {
             data.prev = null;
             currentFiberData.transferred = false;
         }
+
+        if (result.type == RequestType.RAISE) {
+            throw ((RubyException) result.data).toThrowable();
+        }
+
+        return processResultData(context, result);
+    }
+    public IRubyObject raise(ThreadContext context, IRubyObject exception) {
+        Ruby runtime = context.runtime;
+
+        final FiberData data = this.data;
+        if (data.prev != null) throw runtime.newFiberError("double resume");
+
+        if (!alive()) throw runtime.newFiberError("dead fiber called");
+
+        FiberData currentFiberData = context.getFiber().data;
+
+        if (data == currentFiberData) {
+            RubyKernel.raise(context, this, exception);
+        }
+
+        if (data.parent != context.getFiberCurrentThread()) throw runtime.newFiberError("fiber called across threads");
+
+        FiberRequest val = new FiberRequest(exception, RequestType.RAISE);
+
+        data.prev = context.getFiber();
+
+        FiberRequest result = null;
+        try {
+            result = exchangeWithFiber(context, currentFiberData, data, val);
+        } finally {
+            if (result == null) {
+                // exception raised, mark transfer as completed
+                data.prev = null;
+                currentFiberData.transferred = false;
+            }
+        }
+
+        if (result.type == RequestType.RAISE) {
+            throw ((RubyException) result.data).toThrowable();
+        }
+
+        return processResultData(context, result);
     }
     
     @JRubyMethod(meta = true)
@@ -252,7 +405,21 @@ public class ThreadFiber extends RubyObject implements ExecutionContext {
         FiberData currentFiberData = verifyCurrentFiber(context, runtime);
         FiberData prevFiberData = currentFiberData.prev.data;
 
-        return exchangeWithFiber(context, currentFiberData, prevFiberData, value);
+        FiberRequest result = exchangeWithFiber(context, currentFiberData, prevFiberData, new FiberRequest(value, RequestType.DATA));
+
+        if (result.type == RequestType.RAISE) {
+            throw ((RubyException) result.data).toThrowable();
+        }
+
+        return processResultData(context, result);
+    }
+
+    private static IRubyObject processResultData(ThreadContext context, FiberRequest result) {
+        IRubyObject data = result.data;
+
+        if (data == RubyBasicObject.NEVER) return context.nil;
+
+        return data;
     }
 
     @JRubyMethod(meta = true, rest = true)
@@ -267,7 +434,13 @@ public class ThreadFiber extends RubyObject implements ExecutionContext {
         FiberData currentFiberData = verifyCurrentFiber(context, runtime);
         FiberData prevFiberData = currentFiberData.prev.data;
 
-        return exchangeWithFiber(context, currentFiberData, prevFiberData, RubyArray.newArrayNoCopy(runtime, value));
+        FiberRequest result = exchangeWithFiber(context, currentFiberData, prevFiberData, new FiberRequest(RubyArray.newArrayNoCopy(runtime, value), RequestType.DATA));
+
+        if (result.type == RequestType.RAISE) {
+            throw ((RubyException) result.data).toThrowable();
+        }
+
+        return processResultData(context, result);
     }
 
     private static FiberData verifyCurrentFiber(ThreadContext context, Ruby runtime) {
@@ -282,13 +455,13 @@ public class ThreadFiber extends RubyObject implements ExecutionContext {
         return currentFiberData;
     }
 
-    @JRubyMethod
-    public IRubyObject __alive__(ThreadContext context) {
+    @JRubyMethod(name = "alive?")
+    public IRubyObject alive_p(ThreadContext context) {
         return RubyBoolean.newBoolean(context, alive());
     }
     
     @JRubyMethod(meta = true)
-    public static IRubyObject __current__(ThreadContext context, IRubyObject recv) {
+    public static IRubyObject current(ThreadContext context, IRubyObject recv) {
         return context.getFiber();
     }
 
@@ -318,23 +491,24 @@ public class ThreadFiber extends RubyObject implements ExecutionContext {
                     ThreadContext context = runtime.getCurrentContext();
                     context.setFiber(data.fiber.get());
                     context.useRecursionGuardsFrom(data.parent.getContext());
-                    fiberThread.set(context.getThread());
-                    context.getThread().setFiberCurrentThread(data.parent);
+                    RubyThread rubyThread = context.getThread();
+                    fiberThread.set(rubyThread);
+                    rubyThread.setFiberCurrentThread(data.parent);
 
                     Thread thread = Thread.currentThread();
                     String oldName = thread.getName();
                     thread.setName("Fiber thread for block at: " + block.getBody().getFile() + ":" + block.getBody().getLine());
 
                     try {
-                        IRubyObject init = data.queue.pop(context);
+                        FiberRequest init = data.queue.pop(context);
 
                         try {
-                            IRubyObject result;
+                            FiberRequest result;
 
                             if (init == NEVER) {
-                                result = block.yieldSpecific(context);
+                                result = new FiberRequest(block.yieldSpecific(context), RequestType.DATA);
                             } else {
-                                result = block.yieldArray(context, init, null);
+                                result = new FiberRequest(block.yieldArray(context, init.data, null), RequestType.DATA);
                             }
 
                             // Clear ThreadFiber's thread since we're on the way out and need to appear non-alive?
@@ -369,7 +543,7 @@ public class ThreadFiber extends RubyObject implements ExecutionContext {
                         }
                     } catch (RaiseException re) {
                         if (data.prev != null) {
-                            data.prev.thread.raise(re.getException());
+                            data.prev.data.queue.push(context, new FiberRequest(re.getException(), RequestType.RAISE));
                         }
                     } catch (Throwable t) {
                         if (data.prev != null) {
@@ -440,6 +614,119 @@ public class ThreadFiber extends RubyObject implements ExecutionContext {
         }
     }
 
+    @JRubyMethod(name = "blocking?")
+    public IRubyObject blocking_p(ThreadContext context) {
+        return RubyBoolean.newBoolean(context, isBlocking());
+    }
+
+    @JRubyMethod(name = "blocking?", meta = true)
+    public static IRubyObject blocking_p_s(ThreadContext context, IRubyObject self) {
+        boolean blocking = context.getFiber().isBlocking();
+        if (!blocking) return context.fals;
+
+        return RubyFixnum.one(context.runtime);
+    }
+
+    @JRubyMethod(name = "blocking", meta = true)
+    public static IRubyObject blocking(ThreadContext context, IRubyObject self, Block block) {
+        ThreadFiber currentFiber = context.getFiber();
+        boolean blocking = currentFiber.isBlocking();
+
+        // If we are already blocking, this is essentially a no-op:
+        if (currentFiber.isBlocking()) {
+            return block.yieldSpecific(context, currentFiber);
+        }
+
+        try {
+            assert !currentFiber.isBlocking() : "fiber was blocking when it should not have been";
+
+            currentFiber.data.blocking = true;
+
+            // Once the fiber is blocking, and current, we increment the thread blocking state:
+            context.getFiberCurrentThread().incrementBlocking();
+
+            return block.yieldSpecific(context, currentFiber);
+        } finally {
+            // We are no longer blocking:
+            currentFiber.data.blocking = false;
+            context.getFiberCurrentThread().decrementBlocking();
+        }
+    }
+
+    @JRubyMethod(name = "backtrace")
+    public IRubyObject backtrace(ThreadContext context) {
+        return backtrace(context, null, null);
+    }
+
+    @JRubyMethod(name = "backtrace")
+    public IRubyObject backtrace(ThreadContext context, IRubyObject level) {
+        return backtrace(context, level, null);
+    }
+
+    @JRubyMethod(name = "backtrace")
+    public IRubyObject backtrace(ThreadContext context, IRubyObject level, IRubyObject length) {
+        ThreadFiber threadFiber = data.fiber.get();
+
+        if (threadFiber == null) return context.nil;
+
+        return threadFiber.thread.backtrace(context, level, length);
+    }
+
+    @JRubyMethod(name = "backtrace_locations")
+    public IRubyObject backtrace_locations(ThreadContext context) {
+        return backtrace_locations(context, null, null);
+    }
+
+    @JRubyMethod(name = "backtrace_locations")
+    public IRubyObject backtrace_locations(ThreadContext context, IRubyObject level) {
+        return backtrace_locations(context, level, null);
+    }
+
+    @JRubyMethod(name = "backtrace_locations")
+    public IRubyObject backtrace_locations(ThreadContext context, IRubyObject level, IRubyObject length) {
+        ThreadFiber threadFiber = data.fiber.get();
+
+        if (threadFiber == null) return context.nil;
+
+        return threadFiber.thread.backtrace_locations(context, level, length);
+    }
+
+    public static class FiberSchedulerSupport {
+        // MRI: rb_fiber_s_schedule_kw and rb_fiber_s_schedule, kw passes on context
+        @JRubyMethod(name = "schedule", meta = true, rest = true, keywords = true)
+        public static IRubyObject schedule(ThreadContext context, IRubyObject self, IRubyObject[] args, Block block) {
+            RubyThread thread = context.getThread();
+            IRubyObject scheduler = thread.getScheduler();
+            IRubyObject fiber = context.nil;
+
+            if (!scheduler.isNil()) {
+                fiber = scheduler.callMethod(context, "fiber", args, block);
+            } else {
+                throw context.runtime.newRuntimeError("No scheduler is available!");
+            }
+
+            return fiber;
+        }
+
+        // MRI: rb_fiber_s_scheduler
+        @JRubyMethod(name = "scheduler", meta = true)
+        public static IRubyObject get_scheduler(ThreadContext context, IRubyObject self) {
+            return context.getFiberCurrentThread().getScheduler();
+        }
+
+        // MRI: rb_fiber_current_scheduler
+        @JRubyMethod(name = "current_scheduler", meta = true)
+        public static IRubyObject current_scheduler(ThreadContext context, IRubyObject self) {
+            return context.getFiberCurrentThread().getSchedulerCurrent();
+        }
+
+        // MRI: rb_fiber_set_scheduler
+        @JRubyMethod(name = "set_scheduler", meta = true)
+        public static IRubyObject set_scheduler(ThreadContext context, IRubyObject self, IRubyObject scheduler) {
+            return context.getFiberCurrentThread().setFiberScheduler(scheduler);
+        }
+    }
+
     public FiberData getData() {
         return data;
     }
@@ -449,10 +736,11 @@ public class ThreadFiber extends RubyObject implements ExecutionContext {
     }
     
     public static class FiberData {
-        FiberData(FiberQueue queue, RubyThread parent, ThreadFiber fiber) {
+        FiberData(FiberQueue queue, RubyThread parent, ThreadFiber fiber, boolean blocking) {
             this.queue = queue;
             this.parent = parent;
             this.fiber = new WeakReference<ThreadFiber>(fiber);
+            this.blocking = blocking;
         }
 
         public ThreadFiber getPrev() {
@@ -464,8 +752,10 @@ public class ThreadFiber extends RubyObject implements ExecutionContext {
         final RubyThread parent;
         final WeakReference<ThreadFiber> fiber;
         volatile boolean transferred;
+        volatile boolean blocking;
     }
     
     volatile FiberData data;
     volatile RubyThread thread;
+    final boolean root;
 }
