@@ -43,12 +43,6 @@ public class CopyInstr extends OneOperandResultBaseInstr implements FixedArityIn
         return new CopyInstr(d.decodeVariable(), d.decodeOperand());
     }
 
-    @Override
-    public void encode(IRWriterEncoder e) {
-        super.encode(e);
-        e.encode(getSource());
-    }
-
     @Override
     public void visit(IRVisitor visitor) {
         visitor.CopyInstr(this);
