@@ -12,8 +12,8 @@ import java.lang.invoke.MethodType;
 * Created by headius on 10/23/14.
 */
 public class InstanceSuperIterInvokeSite extends ResolvedSuperInvokeSite {
-    public InstanceSuperIterInvokeSite(MethodType type, String name, String splatmapString, String file, int line) {
-        super(type, name, splatmapString, file, line);
+    public InstanceSuperIterInvokeSite(MethodType type, String name, String splatmapString, int flags, String file, int line) {
+        super(type, name, splatmapString, flags, file, line);
     }
 
     @Override
@@ -26,6 +26,7 @@ public class InstanceSuperIterInvokeSite extends ResolvedSuperInvokeSite {
     public IRubyObject invoke(ThreadContext context, IRubyObject caller, IRubyObject self, RubyClass definingModule, IRubyObject[] args, Block block) throws Throwable {
         // TODO: get rid of caller
         // TODO: caching
+        IRRuntimeHelpers.setCallInfo(context, flags);
         return IRRuntimeHelpers.instanceSuperIterSplatArgs(context, self, superName, definingModule, args, block, splatMap);
     }
 }
