@@ -58,6 +58,7 @@ import com.headius.backport9.modules.Modules;
 import org.jcodings.Encoding;
 
 import org.jruby.*;
+import org.jruby.exceptions.NameError;
 import org.jruby.exceptions.TypeError;
 import org.jruby.javasupport.binding.Initializer;
 import org.jruby.javasupport.proxy.JavaProxyClass;
@@ -249,14 +250,30 @@ public class Java implements Library {
         return setProxyClass(runtime, (RubyModule) module, name.asJavaString(), resolveJavaClassArgument(runtime, javaClass));
     }
 
-    public static RubyModule setProxyClass(final Ruby runtime, final RubyModule target,
-                                           final String constName, final Class<?> javaClass) {
+    public static RubyModule setProxyClass(final Ruby runtime, final RubyModule target, final String constName, final Class<?> javaClass) throws NameError {
         final RubyModule proxyClass = getProxyClass(runtime, javaClass);
-
-        target.setConstant(constName, proxyClass);
+        setProxyClass(runtime, target, constName, proxyClass, true);
         return proxyClass;
     }
 
+    private static void setProxyClass(final Ruby runtime, final RubyModule target, final String constName, final RubyModule proxyClass, final boolean validateConstant) {
+        if (constantNotSetOrDifferent(target, constName, proxyClass)) {
+            synchronized (target) { // synchronize to prevent "already initialized constant" warnings with multiple threads
+                if (constantNotSetOrDifferent(target, constName, proxyClass)) {
+                    if (validateConstant) {
+                        target.const_set(runtime.newSymbol(constName), proxyClass); // setConstant would not validate const-name
+                    } else {
+                        target.setConstant(constName, proxyClass);
+                    }
+                }
+            }
+        }
+    }
+
+    private static boolean constantNotSetOrDifferent(final RubyModule target, final String constName, final RubyModule proxyClass) {
+        return !target.constDefinedAt(constName) || !proxyClass.equals(target.getConstant(constName, false));
+    }
+
     /**
      * Same as Java#getInstance(runtime, rawJavaObject, false).
      */
@@ -414,9 +431,14 @@ public class Java implements Library {
 
     @SuppressWarnings("deprecation")
     public static RubyModule getProxyClass(final Ruby runtime, final Class<?> clazz) {
+        return getProxyClass(runtime, clazz, Options.JI_EAGER_CONSTANTS.load());
+    }
+
+    @SuppressWarnings("deprecation")
+    public static RubyModule getProxyClass(final Ruby runtime, final Class<?> clazz, boolean setConstant) {
         RubyModule proxy = runtime.getJavaSupport().getUnfinishedProxy(clazz);
         if (proxy != null) return proxy;
-        return runtime.getJavaSupport().getProxyClassFromCache(clazz);
+        return runtime.getJavaSupport().getProxyClassFromCache(clazz, setConstant);
     }
 
     // expected to handle Java proxy (Ruby) sub-classes as well
@@ -473,7 +495,6 @@ public class Java implements Library {
             proxy.includeModule(extModule);
         }
         Initializer.setupProxyModule(runtime, javaClass, proxy);
-        addToJavaPackageModule(proxy);
     }
 
     private static void generateClassProxy(Ruby runtime, Class<?> clazz, RubyClass proxy, RubyClass superClass) {
@@ -495,7 +516,6 @@ public class Java implements Library {
             } else {
                 proxy.getMetaClass().defineAnnotatedMethods(OldStyleExtensionInherited.class);
             }
-            addToJavaPackageModule(proxy);
         }
         else {
             createProxyClass(runtime, proxy, clazz, superClass, false);
@@ -504,9 +524,6 @@ public class Java implements Library {
             for ( int i = interfaces.length; --i >= 0; ) {
                 proxy.includeModule(getInterfaceModule(runtime, interfaces[i]));
             }
-            if ( Modifier.isPublic(clazz.getModifiers()) ) {
-                addToJavaPackageModule(proxy);
-            }
         }
 
         // JRUBY-1000, fail early when attempting to subclass a final Java class;
@@ -824,9 +841,7 @@ public class Java implements Library {
 
     // package scheme 2: separate module for each full package name, constructed
     // from the camel-cased package segments: Java::JavaLang::Object,
-    private static void addToJavaPackageModule(RubyModule proxyClass) {
-        final Ruby runtime = proxyClass.getRuntime();
-        final Class<?> clazz = (Class<?>)proxyClass.dataGetStruct();
+    static void addToJavaPackageModule(Ruby runtime, Class<?> clazz, RubyModule proxyClass) {
         final String fullName;
         if ( ( fullName = clazz.getName() ) == null ) return;
 
@@ -849,9 +864,8 @@ public class Java implements Library {
 
         if ( parentModule != null && // TODO a Java Ruby class should not validate (as well)
             ( IdUtil.isConstant(className) || parentModule instanceof JavaPackage ) ) {
-            if (parentModule.getConstantAt(className) == null) {
-                parentModule.setConstant(className, proxyClass);
-            }
+            // setConstant without validation since Java class name might be lower-case
+            setProxyClass(runtime, parentModule, className, proxyClass, false);
         }
     }
 
@@ -1074,7 +1088,7 @@ public class Java implements Library {
         } catch (ClassNotFoundException ex) { // used to catch NoClassDefFoundError for whatever reason
             return null;
         }
-        return getProxyClass(runtime, clazz);
+        return getProxyClass(runtime, clazz, true);
     }
 
     private static int mapMajorMinorClassVersionToJavaVersion(String msg, final int offset) {
