@@ -355,10 +355,6 @@ public abstract class JavaLang {
 
         static RubyClass define(final Ruby runtime, final RubyClass proxy) {
             proxy.defineAnnotatedMethods(Number.class);
-
-            proxy.defineAlias("to_int", "longValue");
-            proxy.defineAlias("to_f", "doubleValue");
-
             return proxy;
         }
 
@@ -825,7 +821,7 @@ public abstract class JavaLang {
 
             RubyString buf = inspectPrefix(context, self.getMetaClass());
             RubyStringBuilder.cat(context.runtime, buf, SPACE);
-            buf.cat19(RubyString.newString(context.runtime, str).inspect());
+            buf.catWithCodeRange(RubyString.newString(context.runtime, str).inspect());
             RubyStringBuilder.cat(context.runtime, buf, GT); // >
 
             return buf;
