@@ -43,8 +43,9 @@ import org.jruby.internal.runtime.AbstractIRMethod;
 import org.jruby.internal.runtime.methods.DynamicMethod;
 import org.jruby.java.proxies.ConcreteJavaProxy.NewMethodReified;
 import org.jruby.java.proxies.ConcreteJavaProxy.StaticJCreateMethod;
+import org.jruby.java.proxies.JavaProxy;
+import org.jruby.java.util.ClassUtils;
 import org.jruby.javasupport.Java;
-import org.jruby.javasupport.JavaObject;
 import org.jruby.javasupport.JavaUtil;
 import org.jruby.runtime.ObjectAllocator;
 import org.jruby.runtime.builtin.IRubyObject;
@@ -64,7 +65,6 @@ import java.util.HashSet;
 import java.util.List;
 
 import static org.jruby.javasupport.JavaCallable.inspectParameterTypes;
-import static org.jruby.javasupport.JavaClass.EMPTY_CLASS_ARRAY;
 
 /**
  * Generalized proxy for classes and interfaces.
@@ -82,9 +82,9 @@ import static org.jruby.javasupport.JavaClass.EMPTY_CLASS_ARRAY;
  * @see java.lang.reflect.Proxy
  *
  */
+@JRubyClass(name="Java::JavaProxyClass")
 public class JavaProxyClass extends JavaProxyReflectionObject {
 
-
     private final Class proxyClass;
     private final ArrayList<JavaProxyMethod> methods = new ArrayList<>();
     private final HashMap<String, ArrayList<JavaProxyMethod>> methodMap = new HashMap<>();
@@ -281,8 +281,8 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
             if (returnType == Short.TYPE) return Short.valueOf((short) 0);
             if (returnType == Integer.TYPE) return Integer.valueOf(0);
             if (returnType == Long.TYPE) return Long.valueOf(0L);
-            if (returnType == Float.TYPE) return new Float(0.0f);
-            if (returnType == Double.TYPE) return new Double(0.0);
+            if (returnType == Float.TYPE) return Float.valueOf(0.0f);
+            if (returnType == Double.TYPE) return Double.valueOf(0.0);
 
             return null;
         }
@@ -338,11 +338,11 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
             }
 
             final IRubyObject invokee = args[0];
-            if ( ! ( invokee instanceof JavaObject ) ) {
-                throw runtime.newTypeError("invokee not a java object");
+            if ( ! ( invokee instanceof JavaProxy) ) {
+                throw runtime.newTypeError("not a java proxy: " + (invokee == null ? null : invokee.getClass()));
             }
 
-            Object receiver_value = ((JavaObject) invokee).getValue();
+            Object receiver_value = ((JavaProxy) invokee).getObject();
 
             final Object[] arguments = new Object[ args.length - 1 ];
 
@@ -472,7 +472,7 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
             types.add(type);
         }
 
-        return types.isEmpty() ? EMPTY_CLASS_ARRAY : types.toArray(new Class[types.size()]);
+        return types.isEmpty() ? ClassUtils.EMPTY_CLASS_ARRAY : types.toArray(new Class[types.size()]);
     }
 
     //
@@ -549,28 +549,28 @@ public class JavaProxyClass extends JavaProxyReflectionObject {
      */
     private static final ThreadLocal<Object[]> lookup = new ThreadLocal<>();
 
-    public static Integer addStaticInitLookup(Object... objects) {
+    public static int addStaticInitLookup(Object... objects) {
         // TODO: is this a log or an exception?
         if (objects != null) ensureStaticIntConsumed();
         lookup.set(objects);
-        if (objects == null) return 0;
-        int val = objects.hashCode();
-        return val;
+        return System.identityHashCode(objects); // 0 if null
     }
 
     public static void ensureStaticIntConsumed() {
         if (lookup.get() != null) {
-            throw new IllegalStateException("Thread local class wasn't consumed for class " + ((RubyClass)lookup.get()[1]).getName());
+            throw new IllegalStateException("Thread local class wasn't consumed for: " + lookup.get()[1]);
         }
     }
 
     // used by reified code in RubyClass
-    public static Object[] getStaticInitLookup(int id) {
-        if (lookup.get() == null) throw new IllegalStateException("Thread local class wasn't set up for reification");
-        if (lookup.get().hashCode() != id) throw new IllegalStateException("Thread local class wasn't was reification was expecting");
-        Object[] o = lookup.get();
+    public static Object[] getStaticInitLookup(final int id) {
+        final Object[] objects = lookup.get();
+        if (objects == null) throw new IllegalStateException("Thread local class wasn't set up for reification");
+        if (System.identityHashCode(objects) != id) {
+            throw new IllegalStateException("Thread local class wasn't what reification was expecting: " +  lookup.get()[1]);
+        }
         lookup.set(null);
-        return o;
+        return objects;
     }
 
     public static JavaProxyClass getProxyClass(final Ruby runtime, final RubyClass clazz) {
