@@ -76,7 +76,7 @@ import org.jruby.runtime.scope.ManyVarsDynamicScope;
  * will point to the previous scope of the enclosing module/class (cref).
  * 
  */
-public class StaticScope implements Serializable {
+public class StaticScope implements Serializable, Cloneable {
     public static final int MAX_SPECIALIZED_SIZE = 50;
     private static final long serialVersionUID = 3423852552352498148L;
 
@@ -732,14 +732,7 @@ public class StaticScope implements Serializable {
         this.ivarNames = ivarWrites;
     }
 
-    private boolean isTopScopeEvals() {
-        Ruby runtime = cref.getRuntime();
-        StaticScope scope = this;
-        while (scope != null && scope.type == Type.EVAL) {
-            if (cref != runtime.getTopSelf()) return false;
-            scope = scope.enclosingScope;
-        }
-
-        return scope != null && scope.enclosingScope == null;
+    public boolean isRuby2Keywords() {
+        return irScope.isRuby2Keywords();
     }
 }
