@@ -52,6 +52,9 @@ public class JavaSites {
     public final FiberSites Fiber = new FiberSites();
     public final MonitorSites Monitor = new MonitorSites();
     public final SetSites Set = new SetSites();
+    public final StructSites Struct = new StructSites();
+    public final DigestSites Digest = new DigestSites();
+    public final MethodSites Method = new MethodSites();
 
     public static class BasicObjectSites {
         public final CallSite respond_to = new FunctionalCachingCallSite("respond_to?");
@@ -158,12 +161,9 @@ public class JavaSites {
         public final CallSite op_and = new FunctionalCachingCallSite("&");
         public final CheckedSites to_hash_checked = new CheckedSites("to_hash");
 
-        public final ThreadContext.RecursiveFunctionEx recursive_cmp = new ThreadContext.RecursiveFunctionEx<IRubyObject>() {
-            @Override
-            public IRubyObject call(ThreadContext context, IRubyObject recv, IRubyObject other, boolean recur) {
-                if (recur || !respond_to_cmp.respondsTo(context, other, other)) return context.nil;
-                return cmp.call(context, other, other, recv);
-            }
+        public final ThreadContext.RecursiveFunctionEx<IRubyObject> recursive_cmp = (context, recv, other, recur) -> {
+            if (recur || !respond_to_cmp.respondsTo(context, other, other)) return context.nil;
+            return cmp.call(context, other, other, recv);
         };
     }
 
@@ -177,6 +177,7 @@ public class JavaSites {
     public static class NumericSites {
         public final RespondToCallSite respond_to_coerce = new RespondToCallSite("coerce");
         public final CallSite coerce = new FunctionalCachingCallSite("coerce");
+        public final CheckedSites coerce_checked = new CheckedSites("coerce");
         public final CallSite op_cmp = new FunctionalCachingCallSite("<=>");
         public final CachingCallSite op_minus = new FunctionalCachingCallSite("-");
         public final CallSite op_quo = new FunctionalCachingCallSite("/");
@@ -250,6 +251,7 @@ public class JavaSites {
         public final CallSite op_le = new FunctionalCachingCallSite("<=");
         public final CallSite op_gt = new FunctionalCachingCallSite(">");
         public final CallSite op_lt = new FunctionalCachingCallSite("<");
+        public final CachingCallSite op_eqq = new FunctionalCachingCallSite("==");
         public final CachingCallSite basic_op_lt = new FunctionalCachingCallSite("<");
         public final CachingCallSite basic_op_gt = new FunctionalCachingCallSite(">");
         public final CallSite op_exp_complex = new FunctionalCachingCallSite("**");
@@ -279,6 +281,10 @@ public class JavaSites {
         public final CheckedSites checked_op_or = new CheckedSites("|");
         public final CheckedSites checked_op_xor = new CheckedSites("^");
         public final CallSite op_cmp = new FunctionalCachingCallSite("<=>");
+        public final CallSite op_ge = new FunctionalCachingCallSite(">=");
+        public final CallSite op_le = new FunctionalCachingCallSite("<=");
+        public final CallSite op_gt = new FunctionalCachingCallSite(">");
+        public final CallSite op_lt = new FunctionalCachingCallSite("<");
         public final CallSite fdiv = new FunctionalCachingCallSite("fdiv");
         public final CachingCallSite basic_op_lt = new FunctionalCachingCallSite("<");
         public final CachingCallSite basic_op_gt = new FunctionalCachingCallSite(">");
@@ -298,21 +304,23 @@ public class JavaSites {
         public final CallSite op_le = new FunctionalCachingCallSite("<=");
         public final CallSite op_gt = new FunctionalCachingCallSite(">");
         public final CallSite op_lt = new FunctionalCachingCallSite("<");
-        public final CallSite op_equal = new FunctionalCachingCallSite("==");
+        public final CachingCallSite op_equal = new FunctionalCachingCallSite("==");
         public final RespondToCallSite respond_to_infinite = new RespondToCallSite("infinite?");
         public final CallSite infinite = new FunctionalCachingCallSite("infinite?");
     }
 
+    public static class StructSites {
+        public final CallSite op_equal = new FunctionalCachingCallSite("==");
+        public final CallSite eql = new FunctionalCachingCallSite("eql?");
+    }
+
     public static class TimeSites {
         public final RespondToCallSite respond_to_cmp = new RespondToCallSite("<=>");
         public final CachingCallSite cmp = new FunctionalCachingCallSite("<=>");
 
-        public final ThreadContext.RecursiveFunctionEx recursive_cmp = new ThreadContext.RecursiveFunctionEx<IRubyObject>() {
-            @Override
-            public IRubyObject call(ThreadContext context, IRubyObject recv, IRubyObject other, boolean recur) {
-                if (recur || !respond_to_cmp.respondsTo(context, other, other)) return context.nil;
-                return cmp.call(context, other, other, recv);
-            }
+        public final ThreadContext.RecursiveFunctionEx<IRubyObject> recursive_cmp = (context, recv, other, recur) -> {
+            if (recur || !respond_to_cmp.respondsTo(context, other, other)) return context.nil;
+            return cmp.call(context, other, other, recv);
         };
 
         public final RespondToCallSite respond_to_to_int = new RespondToCallSite("to_int");
@@ -388,11 +396,9 @@ public class JavaSites {
         public final CallSite hash = new FunctionalCachingCallSite("hash");
         public final CallSite op_equal = new FunctionalCachingCallSite("==");
 
-        public final ThreadContext.RecursiveFunctionEx<Ruby> recursive_hash = new ThreadContext.RecursiveFunctionEx<Ruby>() {
-            public IRubyObject call(ThreadContext context, Ruby runtime, IRubyObject obj, boolean recur) {
-                if (recur) return RubyFixnum.zero(runtime);
-                return hash.call(context, obj, obj);
-            }
+        public final ThreadContext.RecursiveFunctionEx<Ruby> recursive_hash = (context, runtime, obj, recur) -> {
+            if (recur) return RubyFixnum.zero(runtime);
+            return hash.call(context, obj, obj);
         };
     }
 
@@ -489,6 +495,7 @@ public class JavaSites {
         public final CallSite glob = new FunctionalCachingCallSite("glob");
         public final CallSite op_plus = new FunctionalCachingCallSite("+");
         public final CallSite sub = new FunctionalCachingCallSite("sub");
+        public final CallSite fnmatch_p = new FunctionalCachingCallSite("fnmatch?");
     }
 
     public static class DateSites {
@@ -520,6 +527,28 @@ public class JavaSites {
     public static class SetSites {
         public final SuperCallSite initialize_dup_super = new SuperCallSite();
         public final SuperCallSite initialize_clone_super = new SuperCallSite();
+        public final CallSite op_equal = new FunctionalCachingCallSite("==");
+        public final CallSite proper_subset = new FunctionalCachingCallSite("proper_subset?");
+        public final CallSite proper_superset = new FunctionalCachingCallSite("proper_superset?");
+        public final CallSite to_a = new FunctionalCachingCallSite("to_a");
+        public final CallSite ary_join = new FunctionalCachingCallSite("join");
+        public final CallSite add = new FunctionalCachingCallSite("add");
+        public final CallSite each_strongly_connected_component = new FunctionalCachingCallSite("each_strongly_connected_component");
+        public final CallSite each = new FunctionalCachingCallSite("each");
+        public final RespondToCallSite respond_to_each = new RespondToCallSite("each");
+        public final CallSite each_entry = new FunctionalCachingCallSite("each_entry");
+        public final RespondToCallSite respond_to_each_entry = new RespondToCallSite("each_entry");
+        public final CallSite merge = new FunctionalCachingCallSite("merge");
+    }
+
+    public static class DigestSites {
+        public final CallSite digest = new FunctionalCachingCallSite("digest");
+        public final CallSite hexdigest = new FunctionalCachingCallSite("hexdigest");
+        public final CallSite digest_length = new FunctionalCachingCallSite("digest_length");
+    }
+
+    public static class MethodSites {
+        public final CallSite curry = new FunctionalCachingCallSite("curry");
     }
 
     public static class CheckedSites {
