@@ -74,9 +74,14 @@ public abstract class CachingCallSite extends CallSite {
         if (cache.typeOk(selfType)) {
             return cache.method.call(context, self, cache.sourceModule, methodName, args);
         }
-        return cacheAndCall(caller, selfType, args, context, self);
+        return cacheAndCall(context, caller, self, selfType, args);
     }
 
+    public IRubyObject fcall(ThreadContext context, IRubyObject self, IRubyObject... args) {
+        return call(context, self, self, args);
+    }
+
+    @Override
     public IRubyObject call(ThreadContext context, IRubyObject caller, IRubyObject self, IRubyObject[] args, Block block) {
         RubyClass selfType = getMetaClass(self);
         // This must be retrieved *once* to avoid racing with other threads.
@@ -84,7 +89,11 @@ public abstract class CachingCallSite extends CallSite {
         if (cache.typeOk(selfType)) {
             return cache.method.call(context, self, cache.sourceModule, methodName, args, block);
         }
-        return cacheAndCall(caller, selfType, block, args, context, self);
+        return cacheAndCall(context, caller, self, selfType, args, block);
+    }
+
+    public IRubyObject fcall(ThreadContext context, IRubyObject self, IRubyObject[] args, Block block) {
+        return call(context, self, self, args, block);
     }
 
     @Override
@@ -97,6 +106,15 @@ public abstract class CachingCallSite extends CallSite {
         }
     }
 
+    public IRubyObject fcallIter(ThreadContext context, IRubyObject self,
+                                IRubyObject[] args, Block block) {
+        try {
+            return call(context, self, self, args, block);
+        } finally {
+            block.escape();
+        }
+    }
+
     @Override
     public final IRubyObject callVarargs(ThreadContext context, IRubyObject caller,
         IRubyObject self, IRubyObject... args) {
@@ -109,6 +127,17 @@ public abstract class CachingCallSite extends CallSite {
         }
     }
 
+    public final IRubyObject fcallVarargs(ThreadContext context,
+                                         IRubyObject self, IRubyObject... args) {
+        switch (args.length) {
+            case 0: return call(context, self, self);
+            case 1: return call(context, self, self, args[0]);
+            case 2: return call(context, self, self, args[0], args[1]);
+            case 3: return call(context, self, self, args[0], args[1], args[2]);
+            default: return call(context, self, self, args);
+        }
+    }
+
     @Override
     public final IRubyObject callVarargs(ThreadContext context, IRubyObject caller,
         IRubyObject self, IRubyObject[] args, Block block) {
@@ -121,6 +150,17 @@ public abstract class CachingCallSite extends CallSite {
         }
     }
 
+    public final IRubyObject fcallVarargs(ThreadContext context,
+                                         IRubyObject self, IRubyObject[] args, Block block) {
+        switch (args.length) {
+            case 0: return call(context, self, self, block);
+            case 1: return call(context, self, self, args[0], block);
+            case 2: return call(context, self, self, args[0], args[1], block);
+            case 3: return call(context, self, self, args[0], args[1], args[2], block);
+            default: return call(context, self, self, args, block);
+        }
+    }
+
     @Override
     public final IRubyObject callVarargsIter(ThreadContext context, IRubyObject caller,
         IRubyObject self, IRubyObject[] args, Block block) {
@@ -133,6 +173,17 @@ public abstract class CachingCallSite extends CallSite {
         }
     }
 
+    public final IRubyObject fcallVarargsIter(ThreadContext context,
+                                             IRubyObject self, IRubyObject[] args, Block block) {
+        switch (args.length) {
+            case 0: return callIter(context, self, self, block);
+            case 1: return callIter(context, self, self, args[0], block);
+            case 2: return callIter(context, self, self, args[0], args[1], block);
+            case 3: return callIter(context, self, self, args[0], args[1], args[2], block);
+            default: return callIter(context, self, self, args, block);
+        }
+    }
+
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject caller, IRubyObject self) {
         RubyClass selfType = getMetaClass(self);
@@ -141,9 +192,14 @@ public abstract class CachingCallSite extends CallSite {
         if (cache.typeOk(selfType)) {
             return cache.method.call(context, self, cache.sourceModule, methodName);
         }
-        return cacheAndCall(caller, selfType, context, self);
+        return cacheAndCall(context, caller, self, selfType);
     }
 
+    public IRubyObject fcall(ThreadContext context, IRubyObject self) {
+        return call(context, self, self);
+    }
+
+    @Override
     public IRubyObject call(ThreadContext context, IRubyObject caller, IRubyObject self, Block block) {
         RubyClass selfType = getMetaClass(self);
         // This must be retrieved *once* to avoid racing with other threads.
@@ -151,7 +207,11 @@ public abstract class CachingCallSite extends CallSite {
         if (cache.typeOk(selfType)) {
             return cache.method.call(context, self, cache.sourceModule, methodName, block);
         }
-        return cacheAndCall(caller, selfType, block, context, self);
+        return cacheAndCall(context, caller, self, selfType, block);
+    }
+
+    public IRubyObject fcall(ThreadContext context, IRubyObject self, Block block) {
+        return call(context, self, self, block);
     }
 
     @Override
@@ -164,6 +224,15 @@ public abstract class CachingCallSite extends CallSite {
         }
     }
 
+    public final IRubyObject fcallIter(ThreadContext context, IRubyObject self,
+                                      Block block) {
+        try {
+            return call(context, self, self, block);
+        } finally {
+            block.escape();
+        }
+    }
+
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject caller, IRubyObject self, IRubyObject arg1) {
         RubyClass selfType = getMetaClass(self);
@@ -172,9 +241,14 @@ public abstract class CachingCallSite extends CallSite {
         if (cache.typeOk(selfType)) {
             return cache.method.call(context, self, cache.sourceModule, methodName, arg1);
         }
-        return cacheAndCall(caller, selfType, context, self, arg1);
+        return cacheAndCall(context, caller, self, selfType, arg1);
     }
 
+    public IRubyObject fcall(ThreadContext context, IRubyObject self, IRubyObject arg1) {
+        return call(context, self, self, arg1);
+    }
+
+    @Override
     public IRubyObject call(ThreadContext context, IRubyObject caller, IRubyObject self, IRubyObject arg1, Block block) {
         RubyClass selfType = getMetaClass(self);
         // This must be retrieved *once* to avoid racing with other threads.
@@ -182,7 +256,11 @@ public abstract class CachingCallSite extends CallSite {
         if (cache.typeOk(selfType)) {
             return cache.method.call(context, self, cache.sourceModule, methodName, arg1, block);
         }
-        return cacheAndCall(caller, selfType, block, context, self, arg1);
+        return cacheAndCall(context, caller, self, selfType, arg1, block);
+    }
+
+    public IRubyObject fcall(ThreadContext context, IRubyObject self, IRubyObject arg1, Block block) {
+        return call(context, self, self, arg1, block);
     }
 
     @Override
@@ -195,6 +273,15 @@ public abstract class CachingCallSite extends CallSite {
         }
     }
 
+    public IRubyObject fcallIter(ThreadContext context, IRubyObject self,
+                                IRubyObject arg1, Block block) {
+        try {
+            return call(context, self, self, arg1, block);
+        } finally {
+            block.escape();
+        }
+    }
+
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject caller, IRubyObject self, IRubyObject arg1, IRubyObject arg2) {
         RubyClass selfType = getMetaClass(self);
@@ -203,9 +290,14 @@ public abstract class CachingCallSite extends CallSite {
         if (cache.typeOk(selfType)) {
             return cache.method.call(context, self, cache.sourceModule, methodName, arg1, arg2);
         }
-        return cacheAndCall(caller, selfType, context, self, arg1, arg2);
+        return cacheAndCall(context, caller, self, selfType, arg1, arg2);
     }
 
+    public IRubyObject fcall(ThreadContext context, IRubyObject self, IRubyObject arg1, IRubyObject arg2) {
+        return call(context, self, self, arg1, arg2);
+    }
+
+    @Override
     public IRubyObject call(ThreadContext context, IRubyObject caller, IRubyObject self, IRubyObject arg1, IRubyObject arg2, Block block) {
         RubyClass selfType = getMetaClass(self);
         // This must be retrieved *once* to avoid racing with other threads.
@@ -213,7 +305,11 @@ public abstract class CachingCallSite extends CallSite {
         if (cache.typeOk(selfType)) {
             return cache.method.call(context, self, cache.sourceModule, methodName, arg1, arg2, block);
         }
-        return cacheAndCall(caller, selfType, block, context, self, arg1, arg2);
+        return cacheAndCall(context, caller, self, selfType, arg1, arg2, block);
+    }
+
+    public IRubyObject fcall(ThreadContext context, IRubyObject self, IRubyObject arg1, IRubyObject arg2, Block block) {
+        return call(context, self, self, arg1, arg2, block);
     }
 
     @Override
@@ -226,6 +322,15 @@ public abstract class CachingCallSite extends CallSite {
         }
     }
 
+    public final IRubyObject fcallIter(ThreadContext context, IRubyObject self,
+                                      IRubyObject arg1, IRubyObject arg2, Block block) {
+        try {
+            return call(context, self, self, arg1, arg2, block);
+        } finally {
+            block.escape();
+        }
+    }
+
     @Override
     public IRubyObject call(ThreadContext context, IRubyObject caller, IRubyObject self, IRubyObject arg1, IRubyObject arg2, IRubyObject arg3) {
         RubyClass selfType = getMetaClass(self);
@@ -234,9 +339,14 @@ public abstract class CachingCallSite extends CallSite {
         if (cache.typeOk(selfType)) {
             return cache.method.call(context, self, cache.sourceModule, methodName, arg1, arg2, arg3);
         }
-        return cacheAndCall(caller, selfType, context, self, arg1, arg2, arg3);
+        return cacheAndCall(context, caller, self, selfType, arg1, arg2, arg3);
     }
 
+    public IRubyObject fcall(ThreadContext context, IRubyObject self, IRubyObject arg1, IRubyObject arg2, IRubyObject arg3) {
+        return call(context, self, self, arg1, arg2, arg3);
+    }
+
+    @Override
     public IRubyObject call(ThreadContext context, IRubyObject caller, IRubyObject self, IRubyObject arg1, IRubyObject arg2, IRubyObject arg3, Block block) {
         RubyClass selfType = getMetaClass(self);
         // This must be retrieved *once* to avoid racing with other threads.
@@ -244,7 +354,11 @@ public abstract class CachingCallSite extends CallSite {
         if (cache.typeOk(selfType)) {
             return cache.method.call(context, self, cache.sourceModule, methodName, arg1, arg2, arg3, block);
         }
-        return cacheAndCall(caller, selfType, block, context, self, arg1, arg2, arg3);
+        return cacheAndCall(context, caller, self, selfType, block, arg1, arg2, arg3);
+    }
+
+    public IRubyObject fcall(ThreadContext context, IRubyObject self, IRubyObject arg1, IRubyObject arg2, IRubyObject arg3, Block block) {
+        return call(context, self, self, arg1, arg2, arg3, block);
     }
 
     @Override
@@ -257,6 +371,15 @@ public abstract class CachingCallSite extends CallSite {
         }
     }
 
+    public final IRubyObject fcallIter(ThreadContext context, IRubyObject self,
+                                      IRubyObject arg1, IRubyObject arg2, IRubyObject arg3, Block block) {
+        try {
+            return call(context, self, self, arg1, arg2, arg3, block);
+        } finally {
+            block.escape();
+        }
+    }
+
     public final CacheEntry retrieveCache(IRubyObject self) {
         RubyClass selfType = getMetaClass(self);
         // This must be retrieved *once* to avoid racing with other threads.
@@ -318,162 +441,66 @@ public abstract class CachingCallSite extends CallSite {
         return entry;
     }
 
-    private IRubyObject cacheAndCall(IRubyObject caller, RubyClass selfType, Block block,
-        IRubyObject[] args, ThreadContext context, IRubyObject self) {
+    protected IRubyObject cacheAndCall(ThreadContext context, IRubyObject caller, IRubyObject self, RubyClass selfType, IRubyObject[] args, Block block) {
+        CacheEntry entry = populateCacheEntry(caller, selfType, context, self);
+        return entry.method.call(context, self, entry.sourceModule, methodName, args, block);
+    }
+
+    protected IRubyObject cacheAndCall(ThreadContext context, IRubyObject caller, IRubyObject self, RubyClass selfType, IRubyObject[] args) {
+        CacheEntry entry = populateCacheEntry(caller, selfType, context, self);
+        return entry.method.call(context, self, entry.sourceModule, methodName, args);
+    }
+
+    protected IRubyObject cacheAndCall(ThreadContext context, IRubyObject caller, IRubyObject self, RubyClass selfType) {
+        CacheEntry entry = populateCacheEntry(caller, selfType, context, self);
+        return entry.method.call(context, self, entry.sourceModule, methodName);
+    }
+
+    protected IRubyObject cacheAndCall(ThreadContext context, IRubyObject caller, IRubyObject self, RubyClass selfType, Block block) {
+        CacheEntry entry = populateCacheEntry(caller, selfType, context, self);
+        return entry.method.call(context, self, entry.sourceModule, methodName, block);
+    }
+
+    protected IRubyObject cacheAndCall(ThreadContext context, IRubyObject caller, IRubyObject self, RubyClass selfType, IRubyObject arg) {
+        CacheEntry entry = populateCacheEntry(caller, selfType, context, self);
+        return entry.method.call(context, self, entry.sourceModule, methodName, arg);
+    }
+
+    protected IRubyObject cacheAndCall(ThreadContext context, IRubyObject caller, IRubyObject self, RubyClass selfType, IRubyObject arg, Block block) {
+        CacheEntry entry = populateCacheEntry(caller, selfType, context, self);
+        return entry.method.call(context, self, entry.sourceModule, methodName, arg, block);
+    }
+
+    protected IRubyObject cacheAndCall(ThreadContext context, IRubyObject caller, IRubyObject self, RubyClass selfType, IRubyObject arg1, IRubyObject arg2) {
+        CacheEntry entry = populateCacheEntry(caller, selfType, context, self);
+        return entry.method.call(context, self, entry.sourceModule, methodName, arg1, arg2);
+    }
+
+    protected IRubyObject cacheAndCall(ThreadContext context, IRubyObject caller, IRubyObject self, RubyClass selfType, IRubyObject arg1, IRubyObject arg2, Block block) {
+        CacheEntry entry = populateCacheEntry(caller, selfType, context, self);
+        return entry.method.call(context, self, entry.sourceModule, methodName, arg1, arg2, block);
+    }
+
+    protected IRubyObject cacheAndCall(ThreadContext context, IRubyObject caller, IRubyObject self, RubyClass selfType, IRubyObject arg1, IRubyObject arg2, IRubyObject arg3) {
+        CacheEntry entry = populateCacheEntry(caller, selfType, context, self);
+        return entry.method.call(context, self, entry.sourceModule, methodName, arg1, arg2, arg3);
+    }
+
+    protected IRubyObject cacheAndCall(ThreadContext context, IRubyObject caller, IRubyObject self, RubyClass selfType, Block block, IRubyObject arg1, IRubyObject arg2, IRubyObject arg3) {
+        CacheEntry entry = populateCacheEntry(caller, selfType, context, self);
+        return entry.method.call(context, self, entry.sourceModule, methodName, arg1, arg2, arg3, block);
+    }
+
+    private CacheEntry populateCacheEntry(IRubyObject caller, RubyClass selfType, ThreadContext context, IRubyObject self) {
         CacheEntry entry = selfType.searchWithCache(methodName);
         DynamicMethod method = entry.method;
+
         if (methodMissing(method, caller)) {
-            return callMethodMissing(context, self, selfType, method, args, block);
+            entry = Helpers.createMethodMissingEntry(context, selfType, callType, method.getVisibility(), entry.token, methodName);
         }
+
         entry = setCache(entry, self);
-        return method.call(context, self, entry.sourceModule, methodName, args, block);
-    }
-
-    private IRubyObject cacheAndCall(IRubyObject caller, RubyClass selfType,
-        IRubyObject[] args, ThreadContext context, IRubyObject self) {
-        CacheEntry entry = selfType.searchWithCache(methodName);
-        DynamicMethod method = entry.method;
-        if (methodMissing(method, caller)) {
-            return callMethodMissing(context, self, selfType, method, args);
-        }
-        entry = setCache(entry, self);
-        return method.call(context, self, entry.sourceModule, methodName, args);
-    }
-
-    private IRubyObject cacheAndCall(IRubyObject caller, RubyClass selfType,
-        ThreadContext context, IRubyObject self) {
-        CacheEntry entry = selfType.searchWithCache(methodName);
-        DynamicMethod method = entry.method;
-        if (methodMissing(method, caller)) {
-            return callMethodMissing(context, self, selfType, method);
-        }
-        entry = setCache(entry, self);
-        return method.call(context, self, entry.sourceModule, methodName);
-    }
-
-    private IRubyObject cacheAndCall(IRubyObject caller, RubyClass selfType, Block block,
-        ThreadContext context, IRubyObject self) {
-        CacheEntry entry = selfType.searchWithCache(methodName);
-        DynamicMethod method = entry.method;
-        if (methodMissing(method, caller)) {
-            return callMethodMissing(context, self, selfType, method, block);
-        }
-        entry = setCache(entry, self);
-        return method.call(context, self, entry.sourceModule, methodName, block);
-    }
-
-    protected IRubyObject cacheAndCall(IRubyObject caller, RubyClass selfType, ThreadContext context, IRubyObject self, IRubyObject arg) {
-        CacheEntry entry = selfType.searchWithCache(methodName);
-        DynamicMethod method = entry.method;
-        if (methodMissing(method, caller)) {
-            return callMethodMissing(context, self, selfType, method, arg);
-        }
-        entry = setCache(entry, self);
-        return method.call(context, self, entry.sourceModule, methodName, arg);
-    }
-
-    private IRubyObject cacheAndCall(IRubyObject caller, RubyClass selfType, Block block,
-        ThreadContext context, IRubyObject self, IRubyObject arg) {
-        CacheEntry entry = selfType.searchWithCache(methodName);
-        DynamicMethod method = entry.method;
-        if (methodMissing(method, caller)) {
-            return callMethodMissing(context, self, selfType, method, arg, block);
-        }
-        entry = setCache(entry, self);
-        return method.call(context, self, entry.sourceModule, methodName, arg, block);
-    }
-
-    protected IRubyObject cacheAndCall(IRubyObject caller, RubyClass selfType, ThreadContext context, IRubyObject self, IRubyObject arg1, IRubyObject arg2) {
-        CacheEntry entry = selfType.searchWithCache(methodName);
-        DynamicMethod method = entry.method;
-        if (methodMissing(method, caller)) {
-            return callMethodMissing(context, self, selfType, method, arg1, arg2);
-        }
-        entry = setCache(entry, self);
-        return method.call(context, self, entry.sourceModule, methodName, arg1, arg2);
-    }
-
-    private IRubyObject cacheAndCall(IRubyObject caller, RubyClass selfType, Block block,
-        ThreadContext context, IRubyObject self, IRubyObject arg1, IRubyObject arg2) {
-        CacheEntry entry = selfType.searchWithCache(methodName);
-        DynamicMethod method = entry.method;
-        if (methodMissing(method, caller)) {
-            return callMethodMissing(context, self, selfType, method, arg1, arg2, block);
-        }
-        entry = setCache(entry, self);
-        return method.call(context, self, entry.sourceModule, methodName, arg1, arg2, block);
-    }
-
-    private IRubyObject cacheAndCall(IRubyObject caller, RubyClass selfType,
-        ThreadContext context, IRubyObject self, IRubyObject arg1, IRubyObject arg2, IRubyObject arg3) {
-        CacheEntry entry = selfType.searchWithCache(methodName);
-        DynamicMethod method = entry.method;
-        if (methodMissing(method, caller)) {
-            return callMethodMissing(context, self, selfType, method, arg1, arg2, arg3);
-        }
-        entry = setCache(entry, self);
-        return method.call(context, self, entry.sourceModule, methodName, arg1, arg2, arg3);
-    }
-
-    private IRubyObject cacheAndCall(IRubyObject caller, RubyClass selfType, Block block,
-        ThreadContext context, IRubyObject self, IRubyObject arg1, IRubyObject arg2, IRubyObject arg3) {
-        CacheEntry entry = selfType.searchWithCache(methodName);
-        DynamicMethod method = entry.method;
-        if (methodMissing(method, caller)) {
-            return callMethodMissing(context, self, selfType, method, arg1, arg2, arg3, block);
-        }
-        entry = setCache(entry, self);
-        return method.call(context, self, entry.sourceModule, methodName, arg1, arg2, arg3, block);
-    }
-
-    protected final IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, RubyClass selfType,
-                                                  DynamicMethod method, IRubyObject[] args) {
-        return Helpers.selectMethodMissing(context, selfType, method.getVisibility(), methodName, callType).call(context, self, selfType, methodName, args, Block.NULL_BLOCK);
-    }
-
-    protected final IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, RubyClass selfType,
-                                                  DynamicMethod method) {
-        return Helpers.selectMethodMissing(context, selfType, method.getVisibility(), methodName, callType).call(context, self, selfType, methodName, Block.NULL_BLOCK);
-    }
-
-    protected final IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, RubyClass selfType,
-                                                  DynamicMethod method, Block block) {
-        return Helpers.selectMethodMissing(context, selfType, method.getVisibility(), methodName, callType).call(context, self, selfType, methodName, block);
-    }
-
-    protected final IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, RubyClass selfType,
-                                                  DynamicMethod method, IRubyObject arg) {
-        return Helpers.selectMethodMissing(context, selfType, method.getVisibility(), methodName, callType).call(context, self, selfType, methodName, arg, Block.NULL_BLOCK);
-    }
-
-    protected final IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, RubyClass selfType,
-                                                  DynamicMethod method, IRubyObject[] args, Block block) {
-        return Helpers.selectMethodMissing(context, selfType, method.getVisibility(), methodName, callType).call(context, self, selfType, methodName, args, block);
-    }
-
-    protected final IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, RubyClass selfType,
-                                                  DynamicMethod method, IRubyObject arg0, Block block) {
-        return Helpers.selectMethodMissing(context, selfType, method.getVisibility(), methodName, callType).call(context, self, selfType, methodName, arg0, block);
-    }
-
-    protected final IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, RubyClass selfType,
-                                                  DynamicMethod method, IRubyObject arg0, IRubyObject arg1) {
-        return Helpers.selectMethodMissing(context, selfType, method.getVisibility(), methodName, callType).call(context, self, selfType, methodName, arg0, arg1, Block.NULL_BLOCK);
-    }
-
-    protected final IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, RubyClass selfType,
-                                                  DynamicMethod method, IRubyObject arg0, IRubyObject arg1, Block block) {
-        return Helpers.selectMethodMissing(context, selfType, method.getVisibility(), methodName, callType).call(context, self, selfType, methodName, arg0, arg1, block);
-    }
-
-    protected final IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, RubyClass selfType,
-                                                  DynamicMethod method, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2) {
-        return Helpers.selectMethodMissing(context, selfType, method.getVisibility(), methodName, callType).call(context, self, selfType, methodName, arg0, arg1, arg2, Block.NULL_BLOCK);
-    }
-
-    protected final IRubyObject callMethodMissing(ThreadContext context, IRubyObject self, RubyClass selfType,
-                                                  DynamicMethod method, IRubyObject arg0, IRubyObject arg1, IRubyObject arg2, Block block) {
-        return Helpers.selectMethodMissing(context, selfType, method.getVisibility(), methodName, callType).call(context, self, self.getMetaClass(), methodName, arg0, arg1, arg2, block);
+        return entry;
     }
 
     protected boolean methodMissing(DynamicMethod method, IRubyObject caller) {
