@@ -14,6 +14,11 @@ public class NClassClassLoader extends ClassLoader implements ClassDefiningClass
         return super.defineClass(name, bytes, 0, bytes.length, ClassDefiningJRubyClassLoader.DEFAULT_DOMAIN);
     }
 
+    @Override
+    public boolean hasDefinedClass(String name) {
+        return super.findLoadedClass(name) != null;
+    }
+
     public boolean isFull() {
         return i >= size;
     }
