@@ -43,6 +43,7 @@ import org.jruby.RubyNumeric;
 import org.jruby.RubyString;
 import org.jruby.RubySymbol;
 import org.jruby.exceptions.RaiseException;
+import org.jruby.java.proxies.JavaProxy;
 import org.jruby.runtime.ClassIndex;
 import org.jruby.runtime.JavaSites;
 import org.jruby.runtime.JavaSites.TypeConverterSites;
@@ -75,6 +76,14 @@ public class TypeConverter {
         return r == null ? handleUncoercibleObject(context.runtime, obj, target, raise) : r;
     }
 
+    // This is essentially convertToType but it is used when we already know the method we are going to call will exist
+    // (as much as we can due to possible race conditions all convert methods have).  This is used in places where we
+    // need to explicitly check for the method (e.g. to_ary) before we actually try and convert the value.
+    public static IRubyObject convertToTypeUnchecked(ThreadContext context, IRubyObject obj, RubyClass target, String convertMethod, boolean raise) {
+        IRubyObject r = obj.callMethod(context, convertMethod);
+        return r == null ? handleUncoercibleObject(context.runtime, obj, target, raise) : r;
+    }
+
     /**
      * Converts this object to type 'targetType' using 'convertMethod' method (MRI: convert_type 1.9).
      *
@@ -143,11 +152,12 @@ public class TypeConverter {
 
     // MRI: rb_to_float - adjusted to handle also Java numbers (non RubyNumeric types)
     public static RubyFloat toFloat(Ruby runtime, IRubyObject obj) {
-        if (obj instanceof RubyNumeric) {
-            return ((RubyNumeric) obj).convertToFloat();
-        }
-        if (obj instanceof RubyString || obj.isNil()) {
-            throw runtime.newTypeError(obj, "Float");
+        if (obj instanceof RubyNumeric) return ((RubyNumeric) obj).convertToFloat();
+        if (obj instanceof RubyString || obj.isNil()) throw runtime.newTypeError(obj, "Float");
+
+        // Java types which can become floats pass to to_f logic and we don't try and match this error string.
+        if (!(obj instanceof JavaProxy) && !obj.getMetaClass().isKindOfModule(runtime.getNumeric())) {
+            throw runtime.newTypeError(str(runtime, "can't convert ", types(runtime, obj.getMetaClass()), " into Float"));
         }
         return (RubyFloat) TypeConverter.convertToType(obj, runtime.getFloat(), "to_f", true);
     }
@@ -223,7 +233,7 @@ public class TypeConverter {
         IRubyObject val = convertToType(obj, target, convertMethod, false);
         if (val.isNil()) return val;
         if (!target.isInstance(val)) {
-            throw newTypeError(obj, target, convertMethod, val);
+            throw newTypeErrorMismatch(obj.getRuntime(), obj, target, convertMethod, val);
         }
         return val;
     }
@@ -242,7 +252,7 @@ public class TypeConverter {
         IRubyObject val = convertToType(context, obj, target, sites, false);
         if (val == context.nil) return val;
         if (!target.isInstance(val)) {
-            throw newTypeError(context.runtime, obj, target, sites.methodName, val);
+            throw newTypeErrorMismatch(context.runtime, obj, target, sites.methodName, val);
         }
         return val;
     }
@@ -275,8 +285,24 @@ public class TypeConverter {
                 className, '#' + methodName + " gives ", types(runtime, val.getType()), ")"));
     }
 
-    // rb_check_to_integer
+    // rb_check_integer_type
     public static IRubyObject checkIntegerType(ThreadContext context, IRubyObject obj) {
+        if (obj instanceof RubyInteger) return obj;
+
+        TypeConverterSites sites = sites(context);
+
+        IRubyObject conv = convertToType(context, obj, context.runtime.getInteger(), sites.to_int_checked, false);
+        if (conv.isNil()) {
+            return context.nil;
+        }
+        if (!(conv instanceof RubyInteger)) {
+            throw newTypeErrorMismatch(context.runtime, obj, context.runtime.getInteger(), "to_int", conv);
+        }
+        return conv;
+    }
+
+    // rb_check_to_integer
+    public static IRubyObject checkToInteger(ThreadContext context, IRubyObject obj) {
         if (obj instanceof RubyFixnum) return obj;
 
         TypeConverterSites sites = sites(context);
@@ -287,8 +313,8 @@ public class TypeConverter {
     }
 
     // rb_check_to_integer
-    public static IRubyObject checkIntegerType(Ruby runtime, IRubyObject obj, String method) {
-        if (method.equals("to_int")) return checkIntegerType(runtime.getCurrentContext(), obj);
+    public static IRubyObject checkToInteger(Ruby runtime, IRubyObject obj, String method) {
+        if (method.equals("to_int")) return checkToInteger(runtime.getCurrentContext(), obj);
 
         if (obj instanceof RubyFixnum) return obj;
 
@@ -418,35 +444,30 @@ public class TypeConverter {
         final Ruby runtime = context.runtime;
         IRubyObject tmp;
 
-        for (;;) {
-            switch (val.getMetaClass().getClassIndex()) {
-                case FLOAT:
-                    if (base != 0) return raiseIntegerBaseError(context, exception);
-                    RubyFloat flote = (RubyFloat) val;
-                    if (!exception && !Double.isFinite(flote.getDoubleValue())) return context.nil;
-                    return RubyNumeric.dbl2ival(context.runtime, flote.getValue());
-                case INTEGER:
-                    if (base != 0) return raiseIntegerBaseError(context, exception);
-                    return val;
-                case STRING:
-                    return RubyNumeric.str2inum(context.runtime, (RubyString) val, base, true, exception);
-                case NIL:
-                    if (base != 0) return raiseIntegerBaseError(context, exception);
-                    if (!exception) return context.nil;
-                    throw context.runtime.newTypeError("can't convert nil into Integer");
-                default: // MRI checks String sub-classes
-                    if (val instanceof RubyString) {
-                        return RubyNumeric.str2inum(context.runtime, (RubyString) val, base, true);
-                    }
-            }
+        if (base != 0) {
+            tmp = TypeConverter.checkStringType(context.runtime, val);
 
-            if (base != 0) {
-                tmp = TypeConverter.checkStringType(context.runtime, val);
-                if (tmp != context.nil) continue;
-                return raiseIntegerBaseError(context, exception);
+            if (!tmp.isNil()) {
+                val = tmp;
+            } else if (!exception) {
+                return context.nil;
+            } else {
+                //raise
+                return raiseIntegerBaseError(context, true);
             }
+        }
 
-            break;
+        if (val instanceof RubyFloat) {
+            RubyFloat f = (RubyFloat) val;
+            if (!exception && !Double.isFinite(f.getDoubleValue())) return context.nil;
+            return RubyNumeric.dbl2ival(context.runtime, f.getValue());
+        } else if (val instanceof RubyInteger) {
+            return val;
+        } else if (val instanceof RubyString) {
+            return RubyNumeric.str2inum(context.runtime, (RubyString) val, base, true, exception);
+        } else if (val.isNil()) {
+            if (!exception) return context.nil;
+            throw context.runtime.newTypeError("can't convert nil into Integer");
         }
 
         try {
@@ -575,4 +596,12 @@ public class TypeConverter {
 
         return false;
     }
+
+    // MRI: rb_bool_expected
+    public static boolean booleanExpected(ThreadContext context, IRubyObject object, String id) {
+        if (object == context.tru) return true;
+        if (object == context.fals) return false;
+
+        throw context.runtime.newArgumentError(str(context.runtime, "true or false is expected as ", context.runtime.newSymbol(id), ": ", object));
+    }
 }
