@@ -58,7 +58,7 @@ default_gems = [
     ['irb', '1.4.2'],
     ['jar-dependencies', '0.4.1'],
     ['jruby-readline', '1.3.7'],
-    ['jruby-openssl', '0.14.3'],
+    ['jruby-openssl', '0.15.0'],
     ['json', '2.7.1'],
     ['logger', '1.5.1'],
     ['mutex_m', '0.1.1'],
@@ -80,7 +80,7 @@ default_gems = [
     ['psych', '5.1.1.1'],
     ['racc', '1.6.0'],
     ['rake-ant', '1.0.6'],
-    ['rdoc', '6.4.0'],
+    ['rdoc', '6.4.1.1'],
     # https://github.com/ruby/readline/issues/5
     # ['readline', '0.0.3'],
     # Will be solved with readline
@@ -112,7 +112,7 @@ default_gems = [
     # ['tmpdir', '0.1.2'],
     ['tsort', '0.1.0'],
     ['un', '0.2.0'],
-    ['uri', '0.12.1'],
+    ['uri', '0.12.2'],
     ['weakref', '0.1.1'],
     # https://github.com/ruby/win32ole/issues/12
     # ['win32ole', '1.8.8'],
@@ -126,7 +126,7 @@ bundled_gems = [
     # ['debug', '1.4.0'],
     ['matrix', '0.4.2'],
     ['minitest', '5.15.0'],
-    ['net-ftp', '0.1.3'],
+    ['net-ftp', '0.3.7'],
     ['net-imap', '0.2.3'],
     ['net-pop', '0.1.1'],
     ['net-smtp', '0.3.1'],
