@@ -4,7 +4,7 @@ gemfile
 
 model.repositories.clear
 
-extension 'org.torquebox.mojo:mavengem-wagon:1.0.3'
+extension 'org.jruby.maven:mavengem-wagon:2.0.2'
 repository :id => :mavengems, :url => 'mavengem:https://rubygems.org'
 
 id 'org.jruby.osgi:gems-bundle', '1.0'
@@ -12,7 +12,7 @@ id 'org.jruby.osgi:gems-bundle', '1.0'
 packaging 'bundle'
 
 # default versions will be overwritten by pom.rb from root directory
-properties( 'jruby.plugins.version' => '1.0.10',
+properties( 'jruby.plugins.version' => '3.0.2',
             # needed bundle plugin
             'polyglot.dump.pom' => 'pom.xml' )
 
