@@ -91,10 +91,10 @@ module JITSpecUtils
       scope.setModule currModule = runtime.top_self.class
     end
 
-    method = org.jruby.ir.IRBuilder.build_root(runtime.getIRManager(), node).scope
+    method = org.jruby.ir.builder.IRBuilder.build_root(runtime.getIRManager(), node).scope
     method.prepareForCompilation
 
-    compiler = org.jruby.ir.targets.JVMVisitor.newForJIT(runtime)
+    compiler = new_visitor(runtime)
     compiled = compiler.compile(method, org.jruby.util.OneShotClassLoader.new(runtime.getJRubyClassLoader()))
     scriptMethod = compiled.getMethod("RUBY$script",
         org.jruby.runtime.ThreadContext.java_class,
@@ -119,6 +119,10 @@ module JITSpecUtils
         descriptors)
   end
 
+  def new_visitor(runtime)
+    org.jruby.ir.targets.JVMVisitor.newForJIT(runtime)
+  end
+
   def compile_run(src, filename, line)
     cls = compile_to_method(src, filename, line - 1) # compiler expects zero-based lines
 
@@ -132,10 +136,21 @@ module JITSpecUtils
   end
 end
 
+module AOTSpecUtils
+  include JITSpecUtils
+
+  def new_visitor(runtime)
+    org.jruby.ir.targets.JVMVisitor.newForAOT(runtime)
+  end
+
+  def self.name; "aot"; end
+end
+
 modes = []
 modes << InterpreterSpecUtils unless (ENV['INTERPRETER_TEST'] == 'false')
 modes << PersistenceSpecUtils unless (ENV['PERSISTENCE_TEST'] == 'false')
 modes << JITSpecUtils unless (ENV['COMPILER_TEST'] == 'false')
+modes << AOTSpecUtils unless (ENV['AOT_TEST'] == 'false')
 
 Block = org.jruby.runtime.Block
 IRubyObject = org.jruby.runtime.builtin.IRubyObject
@@ -155,6 +170,7 @@ modes.each do |mode|
       run("a = {:foo => {:bar => 5.5}}; a") {|result| expect(result).to eq({:foo => {:bar => 5.5}}) }
       run("a = /foo/; a") {|result| expect(result).to eq(/foo/) }
       run("1..2") {|result| expect(result).to eq (1..2) }
+      run("1...2") {|result| expect(result).to eq (1...2) }
       run("1r") {|result| expect(result).to eq (Rational(1, 1))}
       run("1.1r") {|result| expect(result).to eq (Rational(11, 10))}
       run("1i") {|result| expect(result).to eq (Complex(0, 1))}
@@ -164,6 +180,8 @@ modes.each do |mode|
     it "compiles interpolated strings" do
       run('a = "hello#{42}"; a') {|result| expect(result).to eq('hello42') }
       run('i = 1; a = "hello#{i + 42}"; a') {|result| expect(result).to eq("hello43") }
+      # same cases in presence of refinements
+      run('class NoToS; end; module AddToS; refine(NoToS){def to_s; "42"; end}; end; class TryToS; using AddToS; def self.a; "hello#{NoToS.new}"; end; end; TryToS.a') {|result| expect(result).to eq('hello42') }
     end
 
     it "compiles calls" do
@@ -571,6 +589,47 @@ modes.each do |mode|
       end
     end
 
+    it "handles optimized homogeneous case/when" do
+      run('
+        case "a"
+        when "b"
+          fail
+        when "a"
+          1
+        else
+          fail
+        end
+      ') do |result|
+        expect(result).to eq 1
+      end
+
+      run('
+        case :a
+        when :b
+          fail
+        when :a
+          1
+        else
+          fail
+        end
+      ') do |result|
+        expect(result).to eq 1
+      end
+
+      run('
+        case 1
+        when 2
+          fail
+        when 1
+          1
+        else
+          fail
+        end
+      ') do |result|
+        expect(result).to eq 1
+      end
+    end     
+
     it "handles 0-4 arg and splatted whens in a caseless case/when" do
       run('
         case
@@ -763,24 +822,24 @@ modes.each do |mode|
 
     it "assigns the result of a terminated loop to LHS variable" do
       # variable assignment of various types from loop results
-      run("a = while true; break 1; end; a") {|result| expect(result).to eq 1 }
-      run("@a = while true; break 1; end; @a") {|result| expect(result).to eq 1 }
-      run("@@a = while true; break 1; end; @@a") {|result| expect(result).to eq 1 }
-      run("$a = while true; break 1; end; $a") {|result| expect(result).to eq 1 }
-      run("a = until false; break 1; end; a") {|result| expect(result).to eq 1 }
-      run("@a = until false; break 1; end; @a") {|result| expect(result).to eq 1 }
-      run("@@a = until false; break 1; end; @@a") {|result| expect(result).to eq 1 }
-      run("$a = until false; break 1; end; $a") {|result| expect(result).to eq 1 }
+      run("class Loupe; def loupe; a = while true; break 1; end; a; end; end; Loupe.new.loupe") {|result| expect(result).to eq 1 }
+      run("class Loupe; def loupe; @a = while true; break 1; end; @a; end; end; Loupe.new.loupe") {|result| expect(result).to eq 1 }
+      run("class Loupe; def loupe; @@a = while true; break 1; end; @@a; end; end; Loupe.new.loupe") {|result| expect(result).to eq 1 }
+      run("class Loupe; def loupe; $a = while true; break 1; end; $a; end; end; Loupe.new.loupe") {|result| expect(result).to eq 1 }
+      run("class Loupe; def loupe; a = until false; break 1; end; a; end; end; Loupe.new.loupe") {|result| expect(result).to eq 1 }
+      run("class Loupe; def loupe; @a = until false; break 1; end; @a; end; end; Loupe.new.loupe") {|result| expect(result).to eq 1 }
+      run("class Loupe; def loupe; @@a = until false; break 1; end; @@a; end; end; Loupe.new.loupe") {|result| expect(result).to eq 1 }
+      run("class Loupe; def loupe; $a = until false; break 1; end; $a; end; end; Loupe.new.loupe") {|result| expect(result).to eq 1 }
 
       # same assignments but loop is within a begin
-      run("a = begin; while true; break 1; end; end; a") {|result| expect(result).to eq 1 }
-      run("@a = begin; while true; break 1; end; end; @a") {|result| expect(result).to eq 1 }
-      run("@@a = begin; while true; break 1; end; end; @@a") {|result| expect(result).to eq 1 }
-      run("$a = begin; while true; break 1; end; end; $a") {|result| expect(result).to eq 1 }
-      run("a = begin; until false; break 1; end; end; a") {|result| expect(result).to eq 1 }
-      run("@a = begin; until false; break 1; end; end; @a") {|result| expect(result).to eq 1 }
-      run("@@a = begin; until false; break 1; end; end; @@a") {|result| expect(result).to eq 1 }
-      run("$a = begin; until false; break 1; end; end; $a") {|result| expect(result).to eq 1 }
+      run("class Loupe; def loupe; a = begin; while true; break 1; end; end; a; end; end; Loupe.new.loupe") {|result| expect(result).to eq 1 }
+      run("class Loupe; def loupe; @a = begin; while true; break 1; end; end; @a; end; end; Loupe.new.loupe") {|result| expect(result).to eq 1 }
+      run("class Loupe; def loupe; @@a = begin; while true; break 1; end; end; @@a; end; end; Loupe.new.loupe") {|result| expect(result).to eq 1 }
+      run("class Loupe; def loupe; $a = begin; while true; break 1; end; end; $a; end; end; Loupe.new.loupe") {|result| expect(result).to eq 1 }
+      run("class Loupe; def loupe; a = begin; until false; break 1; end; end; a; end; end; Loupe.new.loupe") {|result| expect(result).to eq 1 }
+      run("class Loupe; def loupe; @a = begin; until false; break 1; end; end; @a; end; end; Loupe.new.loupe") {|result| expect(result).to eq 1 }
+      run("class Loupe; def loupe; @@a = begin; until false; break 1; end; end; @@a; end; end; Loupe.new.loupe") {|result| expect(result).to eq 1 }
+      run("class Loupe; def loupe; $a = begin; until false; break 1; end; end; $a; end; end; Loupe.new.loupe") {|result| expect(result).to eq 1 }
 
       # other contexts that require while to preserve stack
       run("1 + while true; break 1; end") {|result| expect(result).to eq 2 }
@@ -883,9 +942,10 @@ modes.each do |mode|
       end
       "){}}.to_not raise_error
 
+      # Enumerator.new without a block
       expect {
         JRuby5871A.new("foo", :each_byte)
-      }.to_not raise_error
+      }.to raise_error(ArgumentError)
 
       expect{run("
       class JRuby5871B < #{enumerable}
@@ -895,9 +955,10 @@ modes.each do |mode|
       end
       "){}}.to_not raise_error
 
+      # Enumerator.new without a block
       expect {
         JRuby5871B.new("foo", :each_byte)
-      }.to_not raise_error
+      }.to raise_error(ArgumentError)
     end
 
     it "allows colon2 const assignment on LHS of masgn" do
@@ -1120,29 +1181,6 @@ modes.each do |mode|
       end
     end
 
-    it "handles circular optional argument assignment" do
-      begin
-        verbose = $VERBOSE
-        $VERBOSE = nil
-
-        run('def foo(a = a, b = b, c = c); [a.inspect, b.inspect, c.inspect]; end; foo') do |x|
-          expect(x).to eq(["nil", "nil", "nil"])
-        end
-        run('def foo; yield; end; foo {|a = a, b = b, c = c|; [a.inspect, b.inspect, c.inspect]}') do |x|
-          expect(x).to eq(["nil", "nil", "nil"])
-        end
-
-        run('def foo(a: a, b: b, c: c); [a.inspect, b.inspect, c.inspect]; end; foo') do |x|
-          expect(x).to eq(["nil", "nil", "nil"])
-        end
-        run('def foo; yield; end; foo {|a: a, b: b, c: c|; [a.inspect, b.inspect, c.inspect]}') do |x|
-          expect(x).to eq(["nil", "nil", "nil"])
-        end
-      ensure
-        $VERBOSE = verbose
-      end
-    end
-
     it "combines optional args and zsuper properly" do
       begin
         verbose = $VERBOSE
@@ -1216,7 +1254,7 @@ modes.each do |mode|
 
     it "handles defined? a.b= forms" do
       run('a = Class.new { attr_writer :b; def []=(_,_); end; def a; [defined? self.b=0, defined? self[0]=0]; end }.new; [a.a, defined? a.b = 0, defined? a[0] = 0]') do |x|
-        expect(x).to eq([["assignment", "assignment"], "assignment", "assignment"])
+        expect(x).to eq([["method", "method"], "method", "method"])
       end
     end
 
@@ -1361,5 +1399,103 @@ modes.each do |mode|
     it "dispatches to Java using a block to implement an interface" do
       run('ary = []; 2.times { java.util.ArrayList.new([1]).forEach { |e| ary << e } }; ary') {|ary| ary.should == [1, 1]}
     end
+
+    it "calls struct field methods" do
+      run('StructTest1 = Struct.new(:foo); st1 = StructTest1.new; st1.foo = 1; st1.foo') {|x| expect(x).to eq(1)}
+      run('StructTest2 = Struct.new(:foo); class StructTest2; def do_foo; self.foo = 1; foo; end; end; StructTest2.new.do_foo') {|x| expect(x).to eq(1)}
+    end
+
+    it "calls aref with string key" do
+      # optimized case for hash receiver
+      run('def foo; {"a" => 1}; end; ary = foo; ary["a"] = 5; ary["a"]') {|val| expect(val).to eq(5)}
+      # normal case for non-hash
+      run('def foo; "abcd"; end; str = foo; str["a"]') {|val| expect(val).to eq("a")}
+      # method_missing case
+      run('o = Object.new; def o.method_missing(sym, a); :ok; end; o["a"]') {|val| expect(val).to eq(:ok)}
+      # failed call site case, calls twice to trigger monomorphic cache in fail path
+      run('10.times.map {o = Object.new; def o.[](a); 1; end; o}.map{|o| 2.times {o["a"]}; o["a"]}.sum') {|val| expect(val).to eq(10)}
+      # failed call site with method_missing
+      run(<<-AREF) {|val| expect(val).to eq(10)}
+        ary = 10.times.map do |i|
+          o = Object.new
+          if i < 9
+            def o.[](a); 1; end
+          else
+            def o.method_missing(sym, a); 1; end;
+          end
+          o
+        end
+        ary.map{|o| o["a"]}.sum
+      AREF
+      # compare_by_identity after usage
+      run('h = {"a" => 1}; val = nil; 2.times { val = h["a"]; h.compare_by_identity }; val') {|val| expect(val).to eq(nil)}
+    end
+
+    it "handles instance super calls with a block" do
+      run(<<-SUPER) {|val| expect(val).to eq(1)}
+        class AInstanceSuper
+          def instance_super
+            yield
+          end
+        end
+        class BInstanceSuper < AInstanceSuper
+          def instance_super
+            super {1}
+          end
+        end
+        BInstanceSuper.new.instance_super
+      SUPER
+    end
+
+    it "handles module super calls with a block" do
+      run(<<-SUPER) {|val| expect(val).to eq(1)}
+          class AModuleSuper
+            def module_super
+              yield
+            end
+          end
+          module BModuleSuper
+            def module_super
+              super {1}
+            end
+          end
+          class CModuleSuper < AModuleSuper
+            include BModuleSuper
+          end
+          CModuleSuper.new.module_super
+      SUPER
+    end
+
+    it "handles class super calls with a block" do
+      run(<<-SUPER) {|val| expect(val).to eq(1)}
+        class AInstanceSuper
+          def self.class_super
+            yield
+          end
+        end
+        class BInstanceSuper < AInstanceSuper
+          def self.class_super
+            super {1}
+          end
+        end
+        BInstanceSuper.class_super
+      SUPER
+    end
+
+    it "handles zsuper calls" do
+      run(<<-SUPER) {|val| expect(val).to eq(1)}
+        class AZSuper
+          def z_super
+            1
+          end
+        end
+        class BZSuper < AZSuper
+          def z_super
+            1.times { super }
+          end
+        end
+        BZSuper.new.z_super
+      SUPER
+    end
   end
 end
