@@ -14,11 +14,11 @@ describe "String#encoding" do
   end
 
   it "returns the given encoding if #force_encoding has been called" do
-    "a".force_encoding(Encoding::SHIFT_JIS).encoding.should == Encoding::SHIFT_JIS
+    "a".dup.force_encoding(Encoding::SHIFT_JIS).encoding.should == Encoding::SHIFT_JIS
   end
 
   it "returns the given encoding if #encode!has been called" do
-    "a".encode!(Encoding::SHIFT_JIS).encoding.should == Encoding::SHIFT_JIS
+    "a".dup.encode!(Encoding::SHIFT_JIS).encoding.should == Encoding::SHIFT_JIS
   end
 end
 
@@ -108,13 +108,13 @@ describe "String#encoding for Strings with \\u escapes" do
   end
 
   it "returns the given encoding if #force_encoding has been called" do
-    "\u{20}".force_encoding(Encoding::SHIFT_JIS).encoding.should == Encoding::SHIFT_JIS
-    "\u{2020}".force_encoding(Encoding::SHIFT_JIS).encoding.should == Encoding::SHIFT_JIS
+    "\u{20}".dup.force_encoding(Encoding::SHIFT_JIS).encoding.should == Encoding::SHIFT_JIS
+    "\u{2020}".dup.force_encoding(Encoding::SHIFT_JIS).encoding.should == Encoding::SHIFT_JIS
   end
 
   it "returns the given encoding if #encode!has been called" do
-    "\u{20}".encode!(Encoding::SHIFT_JIS).encoding.should == Encoding::SHIFT_JIS
-    "\u{2020}".encode!(Encoding::SHIFT_JIS).encoding.should == Encoding::SHIFT_JIS
+    "\u{20}".dup.encode!(Encoding::SHIFT_JIS).encoding.should == Encoding::SHIFT_JIS
+    "\u{2020}".dup.encode!(Encoding::SHIFT_JIS).encoding.should == Encoding::SHIFT_JIS
   end
 end
 
@@ -173,16 +173,12 @@ describe "String#encoding for Strings with \\x escapes" do
   end
 
   it "returns the given encoding if #force_encoding has been called" do
-    x50 = "\x50"
-    x50.force_encoding(Encoding::SHIFT_JIS).encoding.should == Encoding::SHIFT_JIS
-    xD4 = [212].pack('C')
-    xD4.force_encoding(Encoding::ISO_8859_9).encoding.should == Encoding::ISO_8859_9
+    "\x50".dup.force_encoding(Encoding::SHIFT_JIS).encoding.should == Encoding::SHIFT_JIS
+    [212].pack('C').force_encoding(Encoding::ISO_8859_9).encoding.should == Encoding::ISO_8859_9
   end
 
   it "returns the given encoding if #encode!has been called" do
-    x50 = "\x50"
-    x50.encode!(Encoding::SHIFT_JIS).encoding.should == Encoding::SHIFT_JIS
-    x00 = "x\00"
-    x00.encode!(Encoding::UTF_8).encoding.should == Encoding::UTF_8
+    "\x50".dup.encode!(Encoding::SHIFT_JIS).encoding.should == Encoding::SHIFT_JIS
+    "x\00".dup.encode!(Encoding::UTF_8).encoding.should == Encoding::UTF_8
   end
 end
