@@ -1,14 +1,11 @@
-exclude :test_aset_error, "works except JRuby raises RangeError (instead of IndexError) on int overflow"
 exclude :test_collect, "breaks in full interp and JIT due to differences in lambda args processing (#6165)"
 exclude :test_combination_with_callcc, "no callcc"
-exclude :test_count, "needs investigation"
 exclude :test_flatten_with_callcc, "no callcc"
-exclude :test_permutation, "super flaky on travis" if ENV['TRAVIS'].eql?('true')
 exclude :test_permutation_with_callcc, "no callcc"
 exclude :test_product_with_callcc, "no callcc"
 exclude :test_reject_with_callcc, "no callcc"
 exclude :test_repeated_combination_with_callcc, "no callcc"
 exclude :test_repeated_permutation_with_callcc, "no callcc"
-exclude :test_rindex2, "needs investigation"
 exclude :test_sample_random, "gen2 conc modifies ary"
 exclude :test_sort_with_callcc, "no callcc"
+exclude :test_short_heap_array_sort_bang_memory_leak, "no working assert_no_memory_leak method"
