@@ -29,12 +29,12 @@ class TestObjSpace < Test::Unit::TestCase
   end
 
   def test_memsize_of_root_shared_string
-    a = "hello" * 5
+    a = "a" * GC::INTERNAL_CONSTANTS[:RVARGC_MAX_ALLOCATE_SIZE]
     b = a.dup
     c = nil
     ObjectSpace.each_object(String) {|x| break c = x if x == a and x.frozen?}
     rv_size = GC::INTERNAL_CONSTANTS[:RVALUE_SIZE]
-    assert_equal([rv_size, rv_size, 26 + rv_size], [a, b, c].map {|x| ObjectSpace.memsize_of(x)})
+    assert_equal([rv_size, rv_size, a.length + 1 + rv_size], [a, b, c].map {|x| ObjectSpace.memsize_of(x)})
   end
 
   def test_argf_memsize
@@ -106,13 +106,15 @@ class TestObjSpace < Test::Unit::TestCase
   end
 
   def test_memsize_of_iseq
-    iseqw = RubyVM::InstructionSequence.compile('def a; a = :b; end')
-    base_obj_size = ObjectSpace.memsize_of(Object.new)
+    iseqw = RubyVM::InstructionSequence.compile('def a; a = :b; a; end')
+    # Use anonymous class as a basic object size because size of Object.new can be increased
+    base_obj_size = ObjectSpace.memsize_of(Class.new.new)
     assert_operator(ObjectSpace.memsize_of(iseqw), :>, base_obj_size)
   end
 
   def test_reachable_objects_from
-    assert_separately %w[--disable-gem -robjspace], "#{<<-"begin;"}\n#{<<-'end;'}"
+    opts = %w[--disable-gem --disable=frozen-string-literal -robjspace]
+    assert_separately opts, "#{<<-"begin;"}\n#{<<-'end;'}"
     begin;
       assert_equal(nil, ObjectSpace.reachable_objects_from(nil))
       assert_equal([Array, 'a', 'b', 'c'], ObjectSpace.reachable_objects_from(['a', 'b', 'c']))
@@ -163,7 +165,17 @@ class TestObjSpace < Test::Unit::TestCase
     end;
   end
 
+  def test_trace_object_allocations_stop_first
+    assert_separately([], "#{<<~"begin;"}\n#{<<~'end;'}")
+    begin;
+      require "objspace"
+      # Make sure stopping before the tracepoints are initialized doesn't raise. See [Bug #17020]
+      ObjectSpace.trace_object_allocations_stop
+    end;
+  end
+
   def test_trace_object_allocations
+    ObjectSpace.trace_object_allocations_clear # clear object_table to get rid of erroneous detection for c0
     Class.name
     o0 = Object.new
     ObjectSpace.trace_object_allocations{
@@ -232,10 +244,19 @@ class TestObjSpace < Test::Unit::TestCase
     GC.enable
   end
 
+  def test_trace_object_allocations_gc_stress
+    EnvUtil.under_gc_stress do
+      ObjectSpace.trace_object_allocations{
+        proc{}
+      }
+    end
+    assert true # success
+  end
+
   def test_dump_flags
     info = ObjectSpace.dump("foo".freeze)
-    assert_match /"wb_protected":true, "old":true/, info
-    assert_match /"fstring":true/, info
+    assert_match(/"wb_protected":true, "old":true/, info)
+    assert_match(/"fstring":true/, info)
     JSON.parse(info) if defined?(JSON)
   end
 
@@ -267,13 +288,18 @@ class TestObjSpace < Test::Unit::TestCase
 
   def assert_dump_object(info, line)
     loc = caller_locations(1, 1)[0]
-    assert_match /"type":"STRING"/, info
-    assert_match /"embedded":true, "bytesize":11, "value":"hello world", "encoding":"UTF-8"/, info
-    assert_match /"file":"#{Regexp.escape __FILE__}", "line":#{line}/, info
-    assert_match /"method":"#{loc.base_label}"/, info
+    assert_match(/"type":"STRING"/, info)
+    assert_match(/"embedded":true, "bytesize":11, "value":"hello world", "encoding":"UTF-8"/, info)
+    assert_match(/"file":"#{Regexp.escape __FILE__}", "line":#{line}/, info)
+    assert_match(/"method":"#{loc.base_label}"/, info)
     JSON.parse(info) if defined?(JSON)
   end
 
+  def test_dump_control_char
+    assert_include(ObjectSpace.dump("\x0f"), '"value":"\u000f"')
+    assert_include(ObjectSpace.dump("\C-?"), '"value":"\u007f"')
+  end
+
   def test_dump_special_consts
     # [ruby-core:69692] [Bug #11291]
     assert_equal('null', ObjectSpace.dump(nil))
@@ -283,10 +309,31 @@ class TestObjSpace < Test::Unit::TestCase
     assert_equal('{"type":"SYMBOL", "value":"foo"}', ObjectSpace.dump(:foo))
   end
 
+  def test_dump_singleton_class
+    assert_include(ObjectSpace.dump(Object), '"name":"Object"')
+    assert_include(ObjectSpace.dump(Kernel), '"name":"Kernel"')
+    assert_include(ObjectSpace.dump(Object.new.singleton_class), '"real_class_name":"Object"')
+
+    singleton = Object.new.singleton_class
+    singleton_dump = ObjectSpace.dump(singleton)
+    assert_include(singleton_dump, '"singleton":true')
+    if defined?(JSON)
+      assert_equal(Object, singleton.superclass)
+      superclass_address = JSON.parse(ObjectSpace.dump(Object)).fetch('address')
+      assert_equal(superclass_address, JSON.parse(singleton_dump).fetch('superclass'))
+    end
+  end
+
+  def test_dump_special_floats
+    assert_match(/"value":"NaN"/, ObjectSpace.dump(Float::NAN))
+    assert_match(/"value":"Inf"/, ObjectSpace.dump(Float::INFINITY))
+    assert_match(/"value":"\-Inf"/, ObjectSpace.dump(-Float::INFINITY))
+  end
+
   def test_dump_dynamic_symbol
     dump = ObjectSpace.dump(("foobar%x" % rand(0x10000)).to_sym)
-    assert_match /"type":"SYMBOL"/, dump
-    assert_match /"value":"foobar\h+"/, dump
+    assert_match(/"type":"SYMBOL"/, dump)
+    assert_match(/"value":"foobar\h+"/, dump)
   end
 
   def test_dump_includes_imemo_type
@@ -296,8 +343,9 @@ class TestObjSpace < Test::Unit::TestCase
           ObjectSpace.dump_all(output: :stdout)
         end
 
-        dump_my_heap_please
+        p dump_my_heap_please
       end;
+      assert_equal 'nil', output.pop
       heap = output.find_all { |l|
         obj = JSON.parse(l)
         obj['type'] == "IMEMO" && obj['imemo_type']
@@ -313,13 +361,38 @@ class TestObjSpace < Test::Unit::TestCase
           ObjectSpace.dump_all(output: :stdout, full: true)
         end
 
-        dump_my_heap_please
+        p dump_my_heap_please
       end;
+      assert_equal 'nil', output.pop
       heap = output.find_all { |l| JSON.parse(l)['type'] == "NONE" }
       assert_operator heap.length, :>, 0
     end
   end
 
+  def test_dump_all_single_generation
+    assert_in_out_err(%w[-robjspace], "#{<<-"begin;"}\n#{<<-'end;'}") do |output, error|
+      begin;
+        def dump_my_heap_please
+          GC.start
+          ObjectSpace.trace_object_allocations_start
+          gc_gen = GC.count
+          puts gc_gen
+          @obj1 = Object.new
+          GC.start
+          @obj2 = Object.new
+          ObjectSpace.dump_all(output: :stdout, since: gc_gen)
+        end
+
+        p dump_my_heap_please
+      end;
+      assert_equal 'nil', output.pop
+      since = output.shift.to_i
+      assert_operator output.size, :>, 0
+      generations = output.map { |l| JSON.parse(l)["generation"] }.uniq.sort
+      assert_equal [since, since + 1], generations
+    end
+  end
+
   def test_dump_addresses_match_dump_all_addresses
     assert_in_out_err(%w[-robjspace], "#{<<-"begin;"}\n#{<<-'end;'}") do |output, error|
       begin;
@@ -329,8 +402,9 @@ class TestObjSpace < Test::Unit::TestCase
           ObjectSpace.dump_all(output: $stdout)
         end
 
-        dump_my_heap_please
+        p $stdout == dump_my_heap_please
       end;
+      assert_equal 'true', output.pop
       needle = JSON.parse(output.first)
       addr = needle['address']
       found  = output.drop(1).find { |l| JSON.parse(l)['address'] == addr }
@@ -347,8 +421,9 @@ class TestObjSpace < Test::Unit::TestCase
           ObjectSpace.dump_all(output: $stdout)
         end
 
-        dump_my_heap_please
+        p $stdout == dump_my_heap_please
       end;
+      assert_equal 'true', output.pop
       needle = JSON.parse(output.first)
       addr = needle['class']
       found  = output.drop(1).find { |l| JSON.parse(l)['address'] == addr }
@@ -356,6 +431,24 @@ class TestObjSpace < Test::Unit::TestCase
     end
   end
 
+  def test_dump_escapes_method_name
+    method_name = "foo\"bar"
+    klass = Class.new do
+      define_method(method_name) { "TEST STRING" }
+    end
+    ObjectSpace.trace_object_allocations_start
+
+    obj = klass.new.send(method_name)
+
+    dump = ObjectSpace.dump(obj)
+    assert_includes dump, '"method":"foo\"bar"'
+
+    parsed = JSON.parse(dump)
+    assert_equal "foo\"bar", parsed["method"]
+  ensure
+    ObjectSpace.trace_object_allocations_stop
+  end
+
   def test_dump_reference_addresses_match_dump_all_addresses
     assert_in_out_err(%w[-robjspace], "#{<<-"begin;"}\n#{<<-'end;'}") do |output, error|
       begin;
@@ -367,8 +460,9 @@ class TestObjSpace < Test::Unit::TestCase
           ObjectSpace.dump_all(output: $stdout)
         end
 
-        dump_my_heap_please
+        p $stdout == dump_my_heap_please
       end;
+      assert_equal 'true', output.pop
       needle = JSON.parse(output.first)
       addr = needle['address']
       found  = output.drop(1).find { |l| (JSON.parse(l)['references'] || []).include? addr }
@@ -378,8 +472,9 @@ class TestObjSpace < Test::Unit::TestCase
 
   def test_dump_all
     entry = /"bytesize":11, "value":"TEST STRING", "encoding":"UTF-8", "file":"-", "line":4, "method":"dump_my_heap_please", "generation":/
+    opts = %w[--disable-gem --disable=frozen-string-literal -robjspace]
 
-    assert_in_out_err(%w[-robjspace], "#{<<-"begin;"}#{<<-'end;'}") do |output, error|
+    assert_in_out_err(opts, "#{<<-"begin;"}#{<<-'end;'}") do |output, error|
       begin;
         def dump_my_heap_please
           ObjectSpace.trace_object_allocations_start
@@ -388,8 +483,9 @@ class TestObjSpace < Test::Unit::TestCase
           ObjectSpace.dump_all(output: :stdout)
         end
 
-        dump_my_heap_please
+        p dump_my_heap_please
       end;
+      assert_equal 'nil', output.pop
       assert_match(entry, output.grep(/TEST STRING/).join("\n"))
     end
 
@@ -398,16 +494,16 @@ class TestObjSpace < Test::Unit::TestCase
         def dump_my_heap_please
           ObjectSpace.trace_object_allocations_start
           GC.start
-          str = "TEST STRING".force_encoding("UTF-8")
+          (str = "TEST STRING").force_encoding("UTF-8")
           ObjectSpace.dump_all().path
         end
 
         puts dump_my_heap_please
       end;
-      skip if /is not supported/ =~ error
-      skip error unless output
-      assert_match(entry, File.readlines(output).grep(/TEST STRING/).join("\n"))
+      assert_nil(error)
+      dump = File.readlines(output)
       File.unlink(output)
+      assert_match(entry, dump.grep(/TEST STRING/).join("\n"))
     end
 
     if defined?(JSON)
@@ -430,8 +526,8 @@ class TestObjSpace < Test::Unit::TestCase
       puts ObjectSpace.dump(File.allocate)
     RUBY
       assert_nil error
-      assert_match /"type":"FILE"/, output
-      assert_not_match /"fd":/, output
+      assert_match(/"type":"FILE"/, output)
+      assert_not_match(/"fd":/, output)
     end
   end
 
@@ -452,6 +548,20 @@ class TestObjSpace < Test::Unit::TestCase
     assert_operator i, :>, 0
   end
 
+  def test_internal_class_of_on_ast
+    children = ObjectSpace.reachable_objects_from(RubyVM::AbstractSyntaxTree.parse("kadomatsu"))
+    children.each {|child| ObjectSpace.internal_class_of(child).itself} # this used to crash
+  end
+
+  def test_name_error_message
+    begin
+      bar
+    rescue => err
+      _, m = ObjectSpace.reachable_objects_from(err)
+    end
+    assert_equal(m, m.clone)
+  end
+
   def traverse_super_classes klass
     while klass
       klass = ObjectSpace.internal_super_of(klass)
@@ -501,4 +611,25 @@ class TestObjSpace < Test::Unit::TestCase
     assert_equal h[:immortal_symbol], h[:immortal_dynamic_symbol] + h[:immortal_static_symbol], m
     ;;;
   end
+
+  def test_anonymous_class_name
+    assert_not_include ObjectSpace.dump(Class.new), '"name"'
+    assert_not_include ObjectSpace.dump(Module.new), '"name"'
+  end
+
+  def test_objspace_trace
+    assert_in_out_err(%w[-robjspace/trace], "#{<<-"begin;"}\n#{<<-'end;'}") do |out, err|
+      begin;
+        a = "foo"
+        b = "b" + "a" + "r"
+        c = 42
+        p a, b, c
+      end;
+      assert_equal ["objspace/trace is enabled"], err
+      assert_equal 3, out.size
+      assert_equal '"foo" @ -:2', out[0]
+      assert_equal '"bar" @ -:3', out[1]
+      assert_equal '42', out[2]
+    end
+  end
 end
