@@ -85,7 +85,7 @@ class TestGemServer < Gem::TestCase
     end
 
     assert_equal 200, @res.status, @res.body
-    assert_match %r| \d\d:\d\d:\d\d |, @res['date']
+    assert_match %r{ \d\d:\d\d:\d\d }, @res['date']
     assert_equal 'application/octet-stream', @res['content-type']
     assert_equal [['a', Gem::Version.new(2), Gem::Platform::RUBY]],
     Marshal.load(@res.body)
@@ -124,7 +124,7 @@ class TestGemServer < Gem::TestCase
     end
 
     assert_equal 200, @res.status, @res.body
-    assert_match %r| \d\d:\d\d:\d\d |, @res['date']
+    assert_match %r{ \d\d:\d\d:\d\d }, @res['date']
     assert_equal 'application/x-gzip', @res['content-type']
     assert_equal [['a', Gem::Version.new(2), Gem::Platform::RUBY]],
                  Marshal.load(Gem::Util.gunzip(@res.body))
@@ -133,7 +133,7 @@ class TestGemServer < Gem::TestCase
   def test_listen
     util_listen
 
-    capture_io do
+    capture_output do
       @server.listen
     end
 
@@ -143,7 +143,7 @@ class TestGemServer < Gem::TestCase
   def test_listen_addresses
     util_listen
 
-    capture_io do
+    capture_output do
       @server.listen %w[a b]
     end
 
@@ -159,7 +159,7 @@ class TestGemServer < Gem::TestCase
     end
 
     assert_equal 200, @res.status, @res.body
-    assert_match %r| \d\d:\d\d:\d\d |, @res['date']
+    assert_match %r{ \d\d:\d\d:\d\d }, @res['date']
     assert_equal 'application/octet-stream', @res['content-type']
     assert_equal [['a', v('3.a'), Gem::Platform::RUBY]],
                  Marshal.load(@res.body)
@@ -174,7 +174,7 @@ class TestGemServer < Gem::TestCase
     end
 
     assert_equal 200, @res.status, @res.body
-    assert_match %r| \d\d:\d\d:\d\d |, @res['date']
+    assert_match %r{ \d\d:\d\d:\d\d }, @res['date']
     assert_equal 'application/x-gzip', @res['content-type']
     assert_equal [['a', v('3.a'), Gem::Platform::RUBY]],
                  Marshal.load(Gem::Util.gunzip(@res.body))
@@ -220,7 +220,7 @@ class TestGemServer < Gem::TestCase
     @server.quick @req, @res
 
     assert_equal 404, @res.status, @res.body
-    assert_match %r| \d\d:\d\d:\d\d |, @res['date']
+    assert_match %r{ \d\d:\d\d:\d\d }, @res['date']
     assert_equal 'text/plain', @res['content-type']
     assert_equal 'No gems found matching "z-9"', @res.body
     assert_equal 404, @res.status
@@ -242,7 +242,9 @@ class TestGemServer < Gem::TestCase
   end
 
   def test_quick_marshal_a_1_mswin32_gemspec_rz
-    quick_gem 'a', '1' do |s| s.platform = Gem::Platform.local end
+    quick_gem 'a', '1' do |s|
+      s.platform = Gem::Platform.local
+    end
 
     data = StringIO.new "GET /quick/Marshal.#{Gem.marshal_version}/a-1-#{Gem::Platform.local}.gemspec.rz HTTP/1.0\r\n\r\n"
     @req.parse data
@@ -315,7 +317,7 @@ class TestGemServer < Gem::TestCase
     @server.rdoc @req, @res
 
     assert_equal 200, @res.status, @res.body
-    assert_match %r|No documentation found|, @res.body
+    assert_match %r{No documentation found}, @res.body
     assert_equal 'text/html', @res['content-type']
   end
 
@@ -326,7 +328,7 @@ class TestGemServer < Gem::TestCase
     @server.root @req, @res
 
     assert_equal 200, @res.status, @res.body
-    assert_match %r| \d\d:\d\d:\d\d |, @res['date']
+    assert_match %r{ \d\d:\d\d:\d\d }, @res['date']
     assert_equal 'text/html', @res['content-type']
   end
 
@@ -353,7 +355,6 @@ class TestGemServer < Gem::TestCase
     assert_match 'z 9', @res.body
   end
 
-
   def test_xss_homepage_fix_289313
     data = StringIO.new "GET / HTTP/1.0\r\n\r\n"
     dir = "#{@gemhome}2"
@@ -525,7 +526,7 @@ class TestGemServer < Gem::TestCase
     @server.specs @req, @res
 
     assert_equal 200, @res.status, @res.body
-    assert_match %r| \d\d:\d\d:\d\d |, @res['date']
+    assert_match %r{ \d\d:\d\d:\d\d }, @res['date']
     assert_equal 'application/octet-stream', @res['content-type']
 
     assert_equal [['a', Gem::Version.new(1), Gem::Platform::RUBY],
@@ -565,7 +566,7 @@ class TestGemServer < Gem::TestCase
     @server.specs @req, @res
 
     assert_equal 200, @res.status, @res.body
-    assert_match %r| \d\d:\d\d:\d\d |, @res['date']
+    assert_match %r{ \d\d:\d\d:\d\d }, @res['date']
     assert_equal 'application/x-gzip', @res['content-type']
 
     assert_equal [['a', Gem::Version.new(1), Gem::Platform::RUBY],
@@ -587,7 +588,7 @@ class TestGemServer < Gem::TestCase
 
     @server.root @req, @res
 
-    refute_match %r|%3A%2F%2F|, @res.body
+    refute_match %r{%3A%2F%2F}, @res.body
   end
 
   def util_listen
